/*** 
 * \file gre/3dvcgeom.h
 * \brief VECTOR_GEOMETRIC3D algorithms
 *
 * \if NODOC
 * $Id: 3dvcgeom.h_v 1.5 2011/10/20 12:41:47 mju Exp $
 *
 * $Log: 3dvcgeom.h_v $
 * Revision 1.5  2011/10/20 12:41:47  mju
 * Remove redundant hdr.
 *
 * Revision 1.4  2005/10/27 13:08:05  mju
 * Use gre namespace instead of legacy types.
 *
 * Revision 1.1  2005/02/01 17:25:41  vdronov
 * Initial revision
 * \endif
**/ 

#ifndef	INC_GRE_3DVCGEOM_H
#define	INC_GRE_3DVCGEOM_H

#ifndef	INC_GRE_3DGEOMET_H
#include <gre/3dgeomet.h>
#endif

#ifndef	INC_GRE_LRVECTOR_H
#include <gre/lrvector.h>
#endif

namespace GRE {

class VECTOR_GEOMETRIC3D : public GEOMETRIC3D {
	public:

		VECTOR_GEOMETRIC3D (			 							  
			);

		virtual ~VECTOR_GEOMETRIC3D (								//! Destructor
			);

		RVC::VECTOR* GetVector () {return &m_Vector; };
		GRE::LAYER_VECTOR* GetVectorLayer () { return m_Layer; };
		double ComputeScale (const INT32 id, const ELEMTYPE elemtype);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		RVC::VECTOR m_Vector;
		GRE::LAYER_VECTOR *m_Layer;
		DRECT3D ALIGN16(m_Extents);
		SPATREF::COORDREFSYS m_CoordRefSys;
		TRANS2D_MAPGEN m_Transformation;

		VECTOR_GEOMETRIC3D (											//! Copy constructor (not implemented)
			const VECTOR_GEOMETRIC3D& rhs
			);

		VECTOR_GEOMETRIC3D& operator= (								//! Assignment operator (not implemented)
			const VECTOR_GEOMETRIC3D& rhs
			);

		ERRVALUE UpdateExtents (
			TRANS2D_MAPGEN& transformation, 
			DRECT3D& extents
            );

		GEOMETRIC3D* v_CreateInstance (
			) { return new VECTOR_GEOMETRIC3D(); };

		ERRVALUE v_Build (
			const GRE::LAYER *layer
			);

		void v_Free (
			);

		const DRECT3D& v_GetExtents (
			) { return m_Extents; };

		void v_SetCoordRefSys (
			const SPATREF::COORDREFSYS& CoordRefSys
			);

		const SPATREF::COORDREFSYS v_GetCoordRefSys (
			) { return m_CoordRefSys; };

		const TRANS2D_MAPGEN& v_GetTransformationInternalToModel (
			) { return m_Transformation; };

		void OnDrawBegin (	GRE::GEOMETRIC3D_BUILDER* builder, const bool wireframe = false);

		void OnDrawEnd ();


		#endif // GENERATING_DOXYGEN_OUTPUT

	};

}		// End of GRE namespace

#endif



