/**
 * \file gre/lrwms.h
 * \brief GRE LAYER_WMS definitions
 *
 * \if NODOC
 * $Id: lrwms.h_v 1.17 2013/01/16 17:01:40 ldyas Exp $
 *
 * $Log: lrwms.h_v $
 * Revision 1.17  2013/01/16 17:01:40  ldyas
 * added hashighlighted virtual method returning false.
 *
 * Revision 1.16  2013/01/15 23:52:27  ldyas
 * rewinding  to 1.14
 *
 * Revision 1.14  2012/11/01 16:45:17  ldyas
 * make GetUseServerUrl constant
 *
 * Revision 1.13  2012/11/01 16:38:03  ldyas
 * added getter for the use server url
 *
 * Revision 1.12  2012/11/01 14:57:49  ldyas
 * added setuseserver url.
 *
 * Revision 1.11  2011/10/20 19:46:00  mju
 * Remove v_canDrawUsingThreads.
 *
 * Revision 1.10  2011/03/09 17:38:48  ldyas
 * do wms connect outside of thread
 *
 * Revision 1.9  2010/11/19 22:58:36  bcolombe
 * Remove use of DLG_GETOBJECT::ITEM
 *
 * Revision 1.8  2010/08/19 23:03:06  bcolombe
 * Add v_AssignItem
 *
 * Revision 1.7  2010/08/16 13:22:36  mju
 * Only use dlg_getobjectitem hdr if use_mgui.
 *
 * Revision 1.6  2010/08/11 22:01:31  bcolombe
 * Added ability to construct layer form DLG_GETOBJECT::ITEM
 *
 * Revision 1.5  2007/05/07 14:19:13  mju
 * Use mistringlist hdr.
 *
 * Revision 1.4  2007/02/23 21:59:51  mju
 * Add setDftLayer.
 *
 * Revision 1.3  2007/02/19 21:27:34  mju
 * Add dispparm members for transparent pixel detection.
 *
 * Revision 1.2  2007/02/12 21:14:33  mju
 * Add formatGetMap stuff.
 *
 * Revision 1.1  2007/02/02 22:35:12  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_GRE_LRWMS_H
#define	INC_GRE_LRWMS_H

#ifndef	INC_GRE_LRIMAGE_H
	#include <gre/lrimage.h>
#endif

#ifndef	INC_MI32_MISTRINGLIST_H
	#include <mi32/mistringlist.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
namespace SOA {
	namespace WMS {
		class CONNECTION;
		}
	}
namespace SPATMOD {
	namespace IMAGE {
		class PIXEL;
		class PIXELTABLE;
		}
	}

#endif

namespace GRE {
//================================================================================================================

//! Raster Layer class
class LAYER_WMS : public GRE::LAYER_IMAGE {
	public:

		// LAYER_WMS::DISPPARM
		class DISPPARM : public LAYER::DISPPARM {
			public:

				DISPPARM ();

				DISPPARM (
					const DISPPARM& rhs
					);

				virtual ~DISPPARM ();

				DISPPARM& operator= (
					const DISPPARM& rhs
					) { if (this != &rhs) { Copy(rhs); } return (*this); }

				//! Build request URL for "GetMap".
				ERRVALUE BuildGetMapRequest (
					const LAYERDC& layerdc,
					MISTRING& RequestURL,
					LPOINT2D& TgtSize,
					TRANS2D_MAPGEN& TransToCRS
					) const;

				//! Build pixel table for detecting transparent pixels.
				ERRVALUE BuildPixelTableTransparent (
					const COLOR& ColorBG,
					SPATMOD::IMAGE::PIXELTABLE& PixelTable
					) const;

				//! Compute preference for format to use in GetMap request.
				//! @return Preference rating, 0 indicates not supported.
				static int ComputeFormatGetMapPreference (
					const MISTRING& Format
					);

				//! Compute region for current settings.
				ERRVALUE ComputeRegion (
					REGION2D& RegionUsed
					) const;

				//! Get connection to WMS.
				const SOA::WMS::CONNECTION& GetConnection (
					) const { return (*m_pConnection); }

				//! Get coordinate reference system.
				const SPATREF::COORDREFSYS& GetCoordRefSys (
					) const { return (m_CoordRefSys); }

				//! Get background pixel detection setting.
				bool GetDetectBackground (
					) const { return (m_DetectBackground); }

				//! Get black pixel detection setting.
				bool GetDetectBlack (
					) const { return (m_DetectBlack); }

				//! Get white pixel detection setting.
				bool GetDetectWhite (
					) const { return (m_DetectWhite); }

				//! Get format for GetMap request.
				const MISTRING& GetFormatGetMap (
					) const { return (m_FormatGetMap); }

				//! Get layer name list.
				const MISTRINGLIST& GetLayerNameList (
					) const { return (m_LayerNameList); }

				//! Get layer style list.
				const MISTRINGLIST& GetLayerStyleList (
					) const { return (m_LayerStyleList); }

				//! Get URL string for server.
				const MISTRING& GetServerURL (
					) const;

				//! Get transparency (alpha) value.
				UINT8 GetTransparency (
					) const { return (m_Transparency); }

				//! Get whether to override getmap with getcap url
				bool GetUseServerUrl (
					) const { return (m_UseServerUrl); }

				//! Determine if has been validated.
				bool IsValidated (
					) const { return (m_IsValidated); }

				//! Set connection to WMS.
				void SetConnection (
					const SOA::WMS::CONNECTION& connection
					);

				//! Set coordinate reference system.
				void SetCoordRefSys (
					const SPATREF::COORDREFSYS& CoordRefSys
					);

				//! Set whether to detect background pixels if format has no alpha component.
				void SetDetectBackground (
					bool DetectBackground
					) { m_DetectBackground = DetectBackground; }

				//! Set whether to detect black pixels an treat as transparent.
				void SetDetectBlack (
					bool DetectBlack
					) { m_DetectBlack = DetectBlack; }

				//! Set whether to detect white pixels an treat as transparent.
				void SetDetectWhite (
					bool DetectWhite
					) { m_DetectWhite = DetectWhite; }

				//! Set to default format for GetMap request.
				void SetDftFormatGetMap (
					);

				//! Set to default layer for current service.
				void SetDftLayer (
					);

				//! Set format for GetMap request.
				void SetFormatGetMap (
					const MISTRING& Format
					);

				//! Set layer list (names and styles).
				ERRVALUE SetLayerList (
					const MISTRINGLIST& LayerNameList,
					const MISTRINGLIST& LayerStyleList
					);

				//! Get transparency (alpha) value.
				void SetTransparency (
					UINT8 Transparency
					) { m_Transparency = Transparency; }

				//! use get capabilities base url
				//! instead of advertised url 
				//! some wms misadvertise sigh.
				void SetUseServerUrl (
					bool use
					) { m_UseServerUrl = use; }

				MISTRING m_Password;
				MISTRING m_Username;

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT

				SOA::WMS::CONNECTION *m_pConnection;
				MISTRINGLIST m_LayerNameList;
				MISTRINGLIST m_LayerStyleList;
				SPATREF::COORDREFSYS m_CoordRefSys;
				MISTRING m_FormatGetMap;
				UINT8 m_Transparency;
				bool m_DetectBackground;
				bool m_DetectBlack;
				bool m_DetectWhite;
				bool m_IsValidated;
				bool m_UseServerUrl;
				
				void Copy (const DISPPARM&);
				static ERRVALUE SerializerCB_Connection (SERIALIZER& serializer, const SERIALIZERITEM *item, void *vDispParm, SERIALIZERITEM::ACTION action);

				// SERIALIZABLE overrides
				virtual const SERIALIZERITEM* SerialGetItemDef (SERIALIZER& serializer) const;
				virtual ERRVALUE SerialRead (SERIALIZER& serializer);
				virtual ERRVALUE SerialWrite (SERIALIZER& serializer, const char *tagname = 0) const;

				// LAYER::DISPPARM overrides
				virtual ERRVALUE v_Validate ();

				#endif // GENERATING_DOXYGEN_OUTPUT
			};	// End of LAYER_WMS::DISPPARM

		//! Register this layer type
		static void RegisterType (
			);

		//! Method to install SML class, called by internal SML function.
		static void SMLClassInstall (
			SMLCONTEXT *context							//!< SML context
			);

		//! Construct layer and insert into list
		explicit LAYER_WMS (
			GRE::GROUP *group,										//!< Group to contain layer
			CREATEFLAGS createflags = CREATEFLAG_None,	//!< Creation flags
			GRE::LISTPOS listpos = GRE::LISTPOS_Last,		//!< Position in list
			LAYER *reflayer = 0								//!< Reference layer for insertion
			);

		virtual ~LAYER_WMS ();

		//! Retrieve layer-specific display parameters.
		//! @return current parameters
		const DISPPARM& GetDispParm (
			) const {
			return (m_dispparm);
			}

		//! Set layer-specific display parameters.
		//!
		//! Validation will be performed on the specified parameters and
		//! adjustments made to the layer's copy if necessary.  Use of this method
		//! usually result in a ChangeBegin / ChangeEnd message pair.
		ERRVALUE SetDispParm (
			const DISPPARM& dispparm					//!< Parameters to set
			);

	protected:

		DISPPARM m_dispparm;								//!< Layer-specific display parameters

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		typedef LAYER_IMAGE BASECLASS;
		class DLG;

	#ifdef USE_MGUI
		DLG *m_pDlgControlPanel;
	#endif

		static void LegendMsgHandler (GRAPHICTREEITEM_MSG *msg);

	#ifdef USE_MGUI
		void DestroyDlgControlPanel ();
	#endif
		ERRVALUE Draw2D (LAYERDC& layerdc);
		void SetupLegend ();

		// Overrides from LAYER.
		virtual ERRVALUE v_AssignObject (const RVC::OBJITEM& item, MDLGPARENT dlgparent);
	#ifdef USE_MGUI
		virtual int v_ControlPanel (MDLGPARENT dlgparent, UINT32 flags = 0);
	//	ERRVALUE RequestPassword (MDLGPARENT dlgparent);
	#endif
		virtual LAYER* v_Copy (GRE::GROUP *group) const;
		virtual ERRVALUE v_Draw (LAYERDC& layerdc);
		virtual bool v_HasHighlighted () const { return false; };
		virtual bool v_HasNullImageCells () const;
		virtual const char* v_SerialGetTagName () const;
		virtual void v_SetDftName (bool ReplaceExisting);
		virtual ERRVALUE v_UpdateExtents (bool forceupdate);
		
		LAYER_WMS (const LAYER_WMS&);
		LAYER_WMS& operator= (const LAYER_WMS&);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};	// End of LAYER_WMS


//===================================================================================================================
}	// End namespace GRE

#endif	// INC_GRE_LRWMS_H
