/** 
 * \file gre/uiformplaces.h 
 * \brief Definitions for GRE::UI::FORM_PLACES class.
 *
 * \if NODOC
 * $Id: uiformplaces.h_v 1.16 2012/11/08 18:58:54 mju Exp $
 *
 * $Log: uiformplaces.h_v $
 * Revision 1.16  2012/11/08 18:58:54  mju
 * Use coordopzoned and spatref hdrs.
 *
 * Revision 1.15  2012/01/10 13:57:52  mju
 * Remove unnecessary hdrs.
 *
 * Revision 1.14  2011/12/27 16:59:41  mju
 * Use imagelst hdr.
 *
 * Revision 1.13  2011/11/28 17:12:54  mju
 * Fix dox dependency.
 *
 * Revision 1.12  2011/10/20 13:15:42  mju
 * Remove redundant hdrs.
 *
 * Revision 1.11  2011/07/11 19:55:43  bcolombe
 * Moved zoom scale/level to separate tool
 *
 * Revision 1.10  2011/05/06 16:23:40  bcolombe
 * Add LastZoomMode
 *
 * Revision 1.9  2011/05/04 16:09:25  bcolombe
 * Add GetLevelFromCurrentScale
 *
 * Revision 1.8  2011/04/22 21:27:51  bcolombe
 * Added entry for each tileset target
 *
 * Revision 1.7  2011/04/21 19:47:54  bcolombe
 * Changes for zoom to level
 *
 * Revision 1.6  2011/04/18 20:54:39  bcolombe
 * Add selection of tileset for level
 *
 * Revision 1.5  2011/04/08 15:44:07  bcolombe
 * Add zoom to tileset level
 *
 * Revision 1.4  2011/03/11 23:10:25  bcolombe
 * Add QueryButton
 *
 * Revision 1.3  2011/03/03 17:36:45  bcolombe
 * Add method for determining good dft crs
 *
 * Revision 1.1  2011/03/01 22:45:28  bcolombe
 * Initial revision
 *
 * Revision 1.3  2011/02/27 19:26:01  bcolombe
 * Add GetCoordRefSys, SetCenter
 *
 * Revision 1.2  2011/02/25 22:19:38  bcolombe
 * Add zoomto tools
 *
 * Revision 1.1  2011/02/21 21:54:20  bcolombe
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_UIFORMPLACES_H
#define	INC_GRE_UIFORMPLACES_H

#ifndef	INC_MGUI_EDIT_H
#include <mgui/edit.h>
#endif

#ifndef INC_MGUI_FORMLIST_H
#include <mgui/formlist.h>
#endif

#ifndef INC_MGUI_TOOLBAR_H
#include <mgui/toolbar.h>
#endif

#ifndef INC_MGUI_FORMBOOK_H
#include <mgui/formbook.h>
#endif

#ifndef INC_MGUI_TREE_H
#include <mgui/tree.h>
#endif

#ifndef INC_MGUI_IMAGELST_H
#include <mgui/imagelst.h>
#endif

#ifndef INC_MGUI_FORMBTXT_H
#include <mgui/formbtxt.h>
#endif

#ifndef INC_MI32_XML_H
#include <mi32/xml.h>
#endif

#ifndef INC_MI32_COORDOPZONED_H
#include <mi32/coordopzoned.h>
#endif

#ifndef INC_MI32_SPATREF_H
#include <mi32/spatref.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations.
namespace GRE {
	class VIEW;
	class VIEWABLE;
	class GROUP;
	class MSG;
	}
#endif


namespace GRE {
namespace UI {

//===================================================================================================================
//! Form consisting of label and string edit field.
class FORM_PLACES : public MGUI::FORM_COMPOSITE {
	public:
		//! Constructor.
		FORM_PLACES (VIEW* view, VIEWABLE* viewable);

		//! Destructor.
		virtual ~FORM_PLACES ();

		//! Create form with label from resource lookup.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,
			INT32 width,
			INT32 height
			);

		//! Get coordrefsys
		const SPATREF::COORDREFSYS& GetCoordRefSys () { return m_CoordRefSys; }

		//! Get form's pane
		MGUI::LAYOUT_PANE& GetFormPane (
			) { return (GetPane()); }

		//! Set center point to use
		void SetCenter (
			const DPOINT2D& point
			) { m_EditCenterX.SetValue(point.x, false); m_EditCenterY.SetValue(point.y, false); }

		//! Set default coordrefsys to use
		void SetCoordRefSys (
			const SPATREF::COORDREFSYS& crs
			) { m_CoordRefSys = crs; UpdateCRS(); }

		#ifndef GENERATING_DOXYGEN_OUTPUT
		class NameDescPair {
			public:
				char* name;
				char* desc;
			};
		#endif

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		enum CMDID {
			CMDID_ZoomTo = 0,
			CMDID_Bing,
			CMDID_Saved,
			CMDID_Query,
			CMDID_Preferences,
			CMDID_List,
			CMDID_Details,
			CMDID_ShowOutside,
			CMDID_AddView
			};

		CMDID m_CurrentMode;
		int m_pagezoomtomode;
		int m_pagebingmode;
		int m_pagesavedmode;
	
		MGUI::FORM_BOOK m_FormBookMode;

		//----ZOOM TO-----
		static void GRECB_Viewable (const GRE::MSG *msg, void *vcb);

		void AnalyzeCoordRefSys (SPATREF::COORDREFSYS& CoordRefSys, GROUP* group);
		void OnCmdRefSys ();
		void OnZoomToZoomToolbar (MGUI::ID id,bool ToggleOn);
		void OnZoomToZoom ();
		void UpdateCRS();
		void UpdateCRSControls();

		bool m_bCRSChanged;
		SPATREF::COORDREFSYS m_CoordRefSys;
		SPATREF::COORDOP_ZONED m_CoordOpZoned;

		MGUI::CTRL_TOOLBAR m_ToolbarModeZoomTo;
		MGUI::CTRL_GROUPBOX m_GbxCenter;
		MGUI::FORM_BUTTONTEXT m_BtxCenterRefSys;
		MGUI::FORM_EDIT_STRING m_EditCenterMGRS;
		MGUI::FORM_EDIT_NUMBER m_EditCenterY;
		MGUI::FORM_EDIT_NUMBER m_EditCenterX;
		//----ZOOM TO-----

		//----BING MAP SERVICE----
		ERRVALUE DoQuery();
		bool GetCenter (DPOINT2D& center);
		bool GetCenter (DPOINT2D& center, const SPATREF::COORDREFSYS& crs);
		MGUI::FORM_LIST& GetList ();
		void OnAddFolder (MGUI::ID id,bool ToggleOn);
		bool OnClick(int itemnum, int fieldnum, const MGUI::POINT& pt, MGUI::POINTERBUTTON btn, MGUI::KEYSTATE state);
		bool OnDoubleClick(int itemnum, int fieldnum, const MGUI::POINT& pt, MGUI::POINTERBUTTON btn, MGUI::KEYSTATE state);
		bool OnDraw(MGUI::FORM_LIST::CUSTOMDRAW& customdraw);
		void OnMenuPreferences (MGUI::ID id,bool ToggleOn);
		void OnMode(MGUI::ID id,bool ToggleOn);
		void OnQuery();
		void OnQueryToolbar (MGUI::ID id,bool ToggleOn);
		void OnSaveView (MGUI::ID id,bool ToggleOn);
		bool OnToolTip(int itemnum, int fieldnum, MGUI::CTRL_TOOLTIP& tooltip);
		void ZoomToLocation(const DRECT2D& extents);

		MILIST<DRECT2D> m_ExtentsList;
		bool m_ShowDetails;
		bool m_ShowLocationsOutside;
		int m_fieldname;
		int m_fieldentity;
		int m_fieldconfidence;
		int m_fieldvalid;
		int m_fieldcalcmethod;
		int m_fieldmatchcode;
		int m_pagelist;
		int m_pagedetails;

		MGUI::CTRL_PUSHBUTTON m_QueryBtn;
		MGUI::CTRL_EDIT_STRING m_EditString;
		MGUI::CTRL_TOOLBAR m_ToolbarModeBing;
		MGUI::FORM_BOOK m_FormBookLists;
		MGUI::FORM_LIST m_FormList;
		MGUI::FORM_LIST m_FormDetails;
		//----BING MAP SERVICE----

		//----SAVED LOCATIONS----
		void IniRead();
		void IniWrite();
		bool OnTreeClick(MGUI::CTRL_TREE::ITEMHND item, MGUI::CTRL_TREE::HITRESULT hit, const LPOINT2D& pt, MGUI::POINTERBUTTON button, MGUI::KEYSTATE state);
		bool OnTreeClickDown(MGUI::CTRL_TREE::ITEMHND item, MGUI::CTRL_TREE::HITRESULT hit, const LPOINT2D& pt, MGUI::POINTERBUTTON button, MGUI::KEYSTATE state);
		MGUI::CUSTOMDRAW::RETVALUE OnTreeDraw(MGUI::CUSTOMDRAW_TREE& draw);
		void ParseNode(MGUI::CTRL_TREE::ITEMHND parent, XMLNODE* node);

		XMLDOC m_doc;
		XMLDOC m_viewabledoc;

		MGUI::CTRL_TOOLBAR m_ToolbarModeSaved;
		MGUI::CTRL_TREE m_Tree;
		MGUI::IMAGELIST m_ImageList;
		MGUI::CTRL_TREE::ITEMHND m_LayoutHnd;
		MGUI::CTRL_TREE::ITEMHND m_GroupHnd;
		MGUI::CTRL_TREE::ITEMHND m_ViewsHnd;
		//----SAVED LOCATIONS----
		
		VIEW* m_view;
		VIEWABLE* m_viewable;

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

}	// End namespace UI
}	// End namespace GRE

#endif		//	INC_GRE_UIFORMPLACES_H
