/**
 * \file mgui/editsmlparameter.h
 * \brief Form for SML parameters
 *
 * Note: Acutal implementation in qedit.c for now.  It may get moved to its own
 *  module (probably editsmlparameter.c), but currently it shares some data
 *  structures which are only defined in qedit.c
 *
 * \if NODOC
 * $Log: editsmlparameter.h_v $
 * Revision 1.4  2008/09/03 13:16:56  mju
 * Fix doxygen warning.
 *
 * Revision 1.3  2007/05/09 19:20:46  drb
 * Fixed some sml namespace issues
 *
 * Revision 1.2  2007/03/19 13:24:59  mju
 * smlcontext is "struct".
 *
 * Revision 1.1  2007/02/05 21:42:15  dwilliss
 * Initial revision
 *
 * \endif
 **/

#ifndef INC_MGUI_EDITSMLPARAMETER_H
#define INC_MGUI_EDITSMLPARAMETER_H

#ifndef	INC_MGUI_EDIT_H
#include <mgui/ctrl.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
class MISTRING;
class XMLNODE;
class SML::CONTEXT;
#endif

namespace MGUI {

class FORM_EDIT_SML_PARAMETERS : public MGUI::FORM_COMPOSITE {
	public:
		FORM_EDIT_SML_PARAMETERS();
		virtual ~FORM_EDIT_SML_PARAMETERS();


		//! Add an item to the parameters pane.
		//! Must call Create() first.
		void AddItem (
			const XMLNODE* node
			);

		//! Apply the user-entered parameters to an SML::CONTEXT.
		void ApplyParameters (
			SML::CONTEXT* context
			) const;

		//! Create the parameters form given an XML \<parameters\> node
		//! If the node is NULL, nothing will be created.
		void Create (
			LAYOUT_PANE_BASE& ParentPane,
			const XMLNODE* ParametersNode
			);

		//! Return the value of one parameter
		bool GetValue (
			const char* name,
			MISTRING& value
			) const;

		//! Return the value of one parameter
		bool GetValue (
			const char* name,
			double& value
			) const;

		//! Set the value of one parameter,
		//! Form must be created first.
		void SetValue (
			const char* name,
			const MISTRING& value
			);

		//! Set the value of one parameter,
		//! Form must be created first.
		void SetValue (
			const char* name,
			double value
			);

	private:
#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV* m_priv;
#endif

	};

}	// End namespace MGUI


#endif	// INC_MGUI_EDITSMLPARAMETER_H

