/**
 * \file mgui/editurl.h
 * \brief MGUI CTRL_EDIT_URL definitions.
 *
 * \if NODOC
 * $Id: editurl.h_v 1.3 2011/11/22 21:32:39 mju Exp $
 *
 * $Log: editurl.h_v $
 * Revision 1.3  2011/11/22 21:32:39  mju
 * Fix dox dependencies.
 *
 * Revision 1.2  2007/05/07 14:21:34  mju
 * Use mistringlist hdr.
 *
 * Revision 1.1  2007/01/19 15:47:53  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_EDITURL_H
#define	INC_MGUI_EDITURL_H

#ifndef	INC_MGUI_EDITBASE_H
	#include <mgui/editbase.h>
#endif

#ifndef	INC_MI32_MISTRINGLIST_H
	#include <mi32/mistringlist.h>
#endif

namespace MGUI {

//===================================================================================================================

//! Control for allowing user to edit a "URL".
//! Provides for matching against specified list of protocols (eg "http"), as well as prefixing a default
//! protocol if user does not enter one.  Also provides error reporting if user enters unrecognized/nonallowed
//! protocol.
class CTRL_EDIT_URL : public CTRL_EDIT_BASE {
	public:

		//! Constructor.
		CTRL_EDIT_URL (
			bool AddHTTP = true						//!< Include "http" and "https" protocols by default.
			);

		//! Destructor.
		virtual ~CTRL_EDIT_URL (
			);

		//! Add protocol to validation list.
		//! The first added protocol (or "http" if defaults used) will be the default if not specified by user.
		void AddProtocol (
			const char *protocol						//!< Protocol, eg "http", "https", "ftp", etc, not including "://"
			);

		//! Clear to empty string.
		void ClearValue (
			bool notify = true						//!< Call OnChangeValue() if control has already been created
			);

		//! Create control.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,	//!< Parent pane
			int width = 0,								//!< Width in 'typical' characters, minimum if can expand based on sizealign
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedHeight
			);

		//! Get current string value.
		const MISTRING& GetValue (
			);

		//! Read value from INI file.
		//! If Create() has already been performed the control's state will be updated.
		void IniRead (
			INIHANDLE IniHandle,						//!< Handle to INI file to read from, 0 for default
			const char *IniGroup,					//!< INI group to read from
			const char *IniField,					//!< INI field to read from
			bool notify = true						//!< Call OnChangeValue() if control has already been created
			);

		//! Write current value to INI file.
		void IniWrite (
			INIHANDLE IniHandle,						//!< Handle to INI file to read from, 0 for default
			const char *IniGroup,					//!< INI group to read from
			const char *IniField						//!< INI field to read from
			) const;

		//! Set current string value with validation.
		//! OnValidate() will be called and if valid, the string will be updated.
		//! No comparison is made with the current string value.
		void SetValue (
			const MIUNICODE* string,				//!< New string to set
			bool notify = true						//!< Call OnChangeValue() if string actually updated
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		MISTRING m_value;
		MISTRINGLIST m_ValidProtocols;

		bool Validate (MISTRING& string) const;

		virtual void v_UpdateValue (MISTRING& newvalue);

		CTRL_EDIT_URL (const CTRL_EDIT_URL&);
		CTRL_EDIT_URL& operator= (const CTRL_EDIT_URL&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//===================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_EDITURL_H
