/**
 * \file mi32/coordopzoned.h
 * \brief COORDOP_ZONED class definitions.
 *
 * \if NODOC
 * $Id: coordopzoned.h_v 1.1 2012/11/08 18:58:34 mju Exp $
 *
 * $Log: coordopzoned.h_v $
 * Revision 1.1  2012/11/08 18:58:34  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_COORDOPZONED_H
#define	INC_MI32_COORDOPZONED_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

#ifdef GEOMDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif

// Forward declarations.
#ifndef GENERATING_DOXYGEN_OUTPUT
class MISTRING;
namespace SPATREF {
	class COORDREFSYS;
	}
#endif


namespace SPATREF {

//=====================================================================================================================

//! Service for converting between 'zoned' and non-zoned reference systems.
class CLASSLIBEXPORT COORDOP_ZONED {
	public:

		//! Constructor.
		COORDOP_ZONED ();

		//! Copy constructor.
		COORDOP_ZONED (
			const COORDOP_ZONED& rhs
			);

		//! Destructor.
		~COORDOP_ZONED ();

		//! Assignment.
		COORDOP_ZONED& operator= (
			const COORDOP_ZONED& rhs
			);

		//! Convert zone and DPOINT2D to non-zoned CRS
		ERRVALUE ConvertFromZoned (
			const MISTRING& zonestr,					//!< Zone string for point
			const DPOINT2D& ipoint,						//!< Point to convert
			const COORDREFSYS& PointCoordRefSys,	//!< Non-zoned coordinate reference system to convert to
			DPOINT2D& opoint								//!< Point returned
			) const;

		//! Convert zone and DPOINT3D to non-zoned CRS
		ERRVALUE ConvertFromZoned (
			const MISTRING& zonestr,					//!< Zone string for point
			const DPOINT3D& ipoint,						//!< Point to convert
			const COORDREFSYS& PointCoordRefSys,	//!< Non-zoned coordinate reference system to convert to
			DPOINT3D& opoint								//!< Point returned
			) const;

		//! Parse and convert string to non-zoned CRS
		ERRVALUE ConvertFromZoned (
			const MISTRING& string,						//!< String to parse
			const COORDREFSYS& PointCoordRefSys,	//!< Non-zoned coordinate reference system to convert to
			DPOINT2D& opoint								//!< Point returned
			) const;

		//! Convert DPOINT2D to zoned CRS.
		ERRVALUE ConvertToZoned (
			const COORDREFSYS& PointCoordRefSys,	//!< Coordinate reference system to convert from
			const DPOINT2D& ipoint,						//!< Point to convert
			MISTRING& zonestr,							//!< Zone string returned
			DPOINT2D& opoint								//!< Point returned
			) const;

		//! Convert DPOINT3D to zoned CRS.
		ERRVALUE ConvertToZoned (
			const COORDREFSYS& PointCoordRefSys,	//!< Coordinate reference system to convert from
			const DPOINT3D& ipoint,						//!< Point to convert
			MISTRING& zonestr,							//!< Zone string returned
			DPOINT3D& opoint								//!< Point returned
			) const;

		//! Set zoned coordinate reference system.
		ERRVALUE SetCoordRefSys (
			const COORDREFSYS& ZonedCRS				//!< Zoned coordinate reference system
			);

		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
	private:
		PRIV *m_pPriv;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

}	// End namespace SPATREF

#undef	CLASSLIBEXPORT

#endif	// INC_MI32_COORDOPZONED_H
