/**
 * \file mi32/elemtypeset.h 
 * \brief Define ELEMTYPESET class.
 *
 * \if NODOC
 * $Id: elemtypeset.h_v 1.4 2011/10/11 13:21:51 mju Exp $
 *
 * $Log: elemtypeset.h_v $
 * Revision 1.4  2011/10/11 13:21:51  mju
 * Remove unneeded hdr.
 *
 * Revision 1.3  2005/07/15 14:16:32  mju
 * Use memory hdr instead of string hdr.
 *
 * Revision 1.1  2004/07/19 16:32:54  scowan
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_ELEMTYPESET_H
#define	INC_MI32_ELEMTYPESET_H

#ifndef	INC_MI32_ELEMTYPE_H
#include <mi32/elemtype.h>
#endif

#ifndef	INC_MEMORY_H
#include <memory.h>		// Includes defn for memset()
#define	INC_MEMORY_H
#endif

//! \class OBJTYPESET rvc/otypeset.h
//! \brief Class to represent "set" of RVC object types.
class ELEMTYPESET {
	public:

		//! Default constructor, creates empty set.
		ELEMTYPESET (
			) { Clear(); }

		//! Implicit constructor from ELEMTYPE.
		//! Allows a single ELEMTYPE value to be passed wherever ELEMTYPESET is specified.
		ELEMTYPESET (
			ELEMTYPE elemtype
			) { Clear(); operator+=(elemtype); }

		//! Convenience constructor for specifying two elemect types.
		ELEMTYPESET (
			ELEMTYPE elemtype1,
			ELEMTYPE elemtype2
			) { Clear(); operator+=(elemtype1); operator+=(elemtype2); }

		//! Convenience constructor for specifying three elemect types.
		ELEMTYPESET (
			ELEMTYPE elemtype1,
			ELEMTYPE elemtype2,
			ELEMTYPE elemtype3
			) { Clear(); operator+=(elemtype1); operator+=(elemtype2); operator+=(elemtype3); }

		//! Add type to set.
		ELEMTYPESET& operator+= (
			ELEMTYPE rhs
			) { if (rhs == ELEMTYPE_NumTypes) SetAll(); else m_elemtypebits[rhs/32] |= 1 << (rhs % 32); return (*this); }

		//! Add type set to set.
		ELEMTYPESET& operator+= (
			const ELEMTYPESET& rhs
			) { for (INT32 i = 0;(i < (ELEMTYPE_NumTypes+31)/32);++i) m_elemtypebits[i] |= rhs.m_elemtypebits[i]; return (*this); }

		//! Remove type from set.
		ELEMTYPESET& operator-= (
			ELEMTYPE rhs
			) { if (rhs == ELEMTYPE_NumTypes) Clear(); else m_elemtypebits[rhs/32] &= ~(1 << (rhs % 32)); return (*this); }

		//! Clear set.
		void Clear (
			) { memset(m_elemtypebits,0,sizeof(m_elemtypebits)); return; }

		//! Determine if set contains specified elemect type.
		bool Contains (
			ELEMTYPE elemtype
			) const { return ((m_elemtypebits[elemtype/32] & (1 << (elemtype % 32))) != 0); }

		//! Get first elemect type in set.
		//! @return First elemect type or ELEMTYPE_NumTypes if no more in set.
		ELEMTYPE GetFirst (
			) const { return (GetNext(ELEMTYPE_Empty)); }

		//! Get next elemect type in set.
		ELEMTYPE GetNext (
			ELEMTYPE elemtype
			) const {
			do {
				++elemtype;
				if (Contains(elemtype)) break;
				} while (elemtype < ELEMTYPE_NumTypes);
			return (elemtype);
			}

		void SetAll (
			) { memset(m_elemtypebits,255,sizeof(m_elemtypebits)); return; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		UINT32 m_elemtypebits[(ELEMTYPE_NumTypes+31)/32];
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


#endif	//!< INC_RVC_OTYPESET_H
