/**
 * \file mi32/labelframeparms.h
 * \brief LABELFRAMEPARMS definitions
 *
 * \if NODOC
 * $Id: labelframeparms.h_v 1.1 2011/12/23 14:50:46 mju Exp $
 *
 * $Log: labelframeparms.h_v $
 * Revision 1.1  2011/12/23 14:50:46  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_LABELFRAMEPARMS_H
#define	INC_MI32_LABELFRAMEPARMS_H

#ifndef	INC_MI32_ELEMSTYL_H
	#include <mi32/elemstyl.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
class SERIALIZERITEM;
#endif


//! In-memory structure for labels with frames and leaders.  This is the
//! serializable part of the structure.
class LABELFRAMEPARMS {
	public:
		enum FRAMESHAPE {
			FRAMESHAPE_None = 0,
			FRAMESHAPE_Rectangle,
			FRAMESHAPE_RoundedRectangle,
			FRAMESHAPE_Ellipse,
			FRAMESHAPE_Circle
			};

		//! Default constructor
		LABELFRAMEPARMS (
			) :
			m_FrameShape(FRAMESHAPE_None),
			m_Margin(0.0),
			m_bCartoonBalloonLeaders(false),
			m_TMargin(0.0),
			m_BMargin(0.0),
			m_LMargin(0.0),
			m_RMargin(0.0),
			m_Arrow(0)
			{ }
		
		//! Copy constructor.
		LABELFRAMEPARMS (									
			const LABELFRAMEPARMS& rhs
			) { Copy(rhs); }

		//! Destructor.
		~LABELFRAMEPARMS() {}
		
		//! Assignment 
		LABELFRAMEPARMS& operator= (					
			const LABELFRAMEPARMS& rhs
			) { if (this != &rhs) { Copy(rhs); } return (*this); }

		static const SERIALIZERITEM* SerialGetItemDef ();

		//! Helper method for copying.
		void Copy (									
			const LABELFRAMEPARMS& rhs
			);

		bool GetCartoonBalloonLeaders () const
			{ return m_bCartoonBalloonLeaders; }

		FRAMESHAPE GetFrameShape () const
			{ return m_FrameShape; }

		const POLYSTYLE& GetFrameStyle (
			) const {
			return m_FrameStyle;
			}

		const LINESTYLE& GetLeaderStyle (
			) const {
			return m_LeaderStyle;
			}

		// Begin phased out, but old data will have it
		double GetMargin (
			) const {
			return m_Margin;
			}

		double GetTopMargin (
			) const {
			return m_TMargin + m_Margin;
			}

		double GetBottomMargin (
			) const {
			return m_BMargin + m_Margin;
			}

		double GetLeftMargin (
			) const {
			return m_LMargin + m_Margin;
			}

		double GetRightMargin (
			) const {
			return m_RMargin + m_Margin;
			}

		void SetCartoonBalloonLeaders (
			bool value
			) {
			m_bCartoonBalloonLeaders = value;
			}

		void SetFrameShape (
			FRAMESHAPE shape
			) {
			m_FrameShape = shape;
			}

		void SetFrameStyle (
			const POLYSTYLE& style
			) {
			m_FrameStyle = style;
			}

		void SetLeaderStyle (
			const LINESTYLE& style
			) {
			m_LeaderStyle = style;
			}

		void SetMargin (
			double margin
			) {
			m_Margin = margin;
			}
		
		void SetTopMargin (
			double margin
			) {
			m_TMargin = margin;
			}
		
		void SetBottomMargin (
			double margin
			) {
			m_BMargin = margin;
			}
		
		void SetLeftMargin (
			double margin
			) {
			m_LMargin = margin;
			}
		
		void SetRightMargin (
			double margin
			) {
			m_RMargin = margin;
			}

		bool UsesTransparency (
			) const {
			if (m_FrameShape != FRAMESHAPE_None && m_FrameStyle.UsesTransparency()) return (true);
			return (m_LeaderStyle.UsesTransparency());
			}

	private:
   	#ifndef GENERATING_DOXYGEN_OUTPUT
		POLYSTYLE m_FrameStyle;
		LINESTYLE m_LeaderStyle;
		FRAMESHAPE m_FrameShape;
		double m_Margin;			//!< Units: % of font ascent 
		bool m_bCartoonBalloonLeaders;
		double m_TMargin;			//!< Units: % of font ascent
		double m_BMargin;			//!< Units: % of font ascent
		double m_LMargin;			//!< Units: % of font ascent
		double m_RMargin;			//!< Units: % of font ascent
		int m_Arrow;

		friend class SML_LABELFRAMEPARMS;	// SML Implementation 
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#endif	// INC_MI32_LABELFRAMEPARMS_H
