/**
 * \file mi32/lidarclasschanges.h
 * \brief LIDAR::CLASSCHANGES definitions
 *
 * \if NODOC
 * $Id: lidarclasschanges.h_v 1.1 2012/10/11 19:10:47 mju Exp $
 *
 * $Log: lidarclasschanges.h_v $
 * Revision 1.1  2012/10/11 19:10:47  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_LIDARCLASSCHANGES_H
#define	INC_MI32_LIDARCLASSCHANGES_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

#ifndef	GENERATING_DOXYGEN_OUTPUT
// Forward declarations;
class BITSET;
class DATETIME;
class FILEPATH;
namespace LIDAR {
	class CLASSDESCSET;
	namespace LAS {
		class DATASET;
		}
	}
#endif

namespace LIDAR {

//=====================================================================================================================

//! Container for classification change statistics.
class CLASSCHANGES {
	public:

		//! Constructor.
		CLASSCHANGES ();

		//! Destructor.
		~CLASSCHANGES ();

		//! Get count of classes for old/new combination.
		UINT64 GetClassCount (
			UINT16 ClassNumOld,				//!< Old class number
			UINT16 ClassNumNew				//!< New class number
			) const;

		//! Get class description set.
		const CLASSDESCSET& GetClassDescSet () const;

		//! Get set of new classes used.
		const BITSET& GetClassSetNew () const;

		//! Get set of old classes used.
		const BITSET& GetClassSetOld () const;

		//! Get timestamp for new classification.
		const DATETIME& GetTimeStampNew () const;

		//! Get timestamp for old classification.
		const DATETIME& GetTimeStampOld () const;

		//! Get total count for new class.
		UINT64 GetTotalCountNew (
			UINT16 ClassNumNew				//!< New class number
			) const;

		//! Get total count for old class.
		UINT64 GetTotalCountOld (
			UINT16 ClassNumOld				//!< Old class number
			) const;

		//! Save report to file.
		ERRVALUE SaveReport (
			const FILEPATH& filepath		//!< Path to (HTML) file to save as
			) const;

		//! Scan dataset for changes.
		ERRVALUE Scan (
			LAS::DATASET& dataset,			//!< Dataset to scan
			const DATETIME& TimeStampOld	//!< Old classification timestamp
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;

		CLASSCHANGES (const CLASSCHANGES&);
		CLASSCHANGES& operator= (const CLASSCHANGES&);
		#endif
	};

//=====================================================================================================================

}	// End namespace LIDAR

#endif	// INC_MI32_LIDARCLASSCHANGES_H
