/**
 * \file mi32/mg2cmsui.h
 * \brief User Interface for selecting ICM settings
 *
 * Include file for User Interface for ICM Color Management classes
 *
 * \if NODOC
 * $Log: mg2cmsui.h_v $
 * Revision 1.9  2011/12/28 22:29:12  mju
 * Remove redundant hdrs.
 *
 * Revision 1.8  2004/03/05 23:05:08  dwilliss
 * made destructors virtual
 *
 * Revision 1.4  2003/06/13 20:28:37  dwilliss
 * Added definition for printer ICM pane
 *
 * Revision 1.1  2003/06/04 17:23:31  dwilliss
 * Initial revision
 * \endif
**/

#ifndef INC_MI32_MG2CMSUI_H
#define INC_MI32_MG2CMSUI_H

#ifndef INC_MI32_MG2CMD_H
#include <mi32/mg2cms.h>
#endif

#ifndef INC_MGUI_COMBOBOX_H
#include <mgui/combobox.h>
#endif


class ICMPANE {
	public:
		ICMPANE();
		virtual ~ICMPANE();

		ERRVALUE Init (
			MGUI::LAYOUT_PANE_BASE& pane
			) {
			return (v_Init(pane));
			}

#ifdef X_NATIVE
		//! Temporary version until the whole View Preferences dialog is
		//! rewritten to use MGUI
		ERRVALUE Init(Widget form);
#endif

	private:
   	#ifndef GENERATING_DOXYGEN_OUTPUT
#ifdef X_NATIVE
		MGUI::LAYOUT_PANE_ROOT m_RootPane;
#endif

		virtual ERRVALUE v_Init(MGUI::LAYOUT_PANE_BASE& pane) = 0;
		#endif // GENERATING_DOXYGEN_OUTPUT

	};


class RGBICMPANE : public ICMPANE {
	public:
		RGBICMPANE();
		virtual ~RGBICMPANE();

		ERRVALUE SaveSettings (
			);

	private:
   	#ifndef GENERATING_DOXYGEN_OUTPUT

		class PRIV;

		PRIV* m_pPriv;

		virtual ERRVALUE v_Init(MGUI::LAYOUT_PANE_BASE& pane);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};


class PRINTERICMPANE : public ICMPANE {
	public:
		PRINTERICMPANE();
		virtual ~PRINTERICMPANE();

		void GetParms(
			MGD::PROFILEPARMS_PRINTER&
			) const;

		void SetParms(
			const MGD::PROFILEPARMS_PRINTER&
			);

	private:
   	#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;

		PRIV* m_pPriv;

		virtual ERRVALUE v_Init(MGUI::LAYOUT_PANE_BASE& pane);
		#endif // GENERATING_DOXYGEN_OUTPUT
		
	};



#endif // INC_MI32_MG2CMSUI_H
