/**
 * \file mi32/misystem.h
 * \brief MISYSTEM class definition
 *
 * \if NODOC
 * $Id: misystem.h_v 1.44 2012/11/08 22:11:33 mju Exp $
 *
 * $Log: misystem.h_v $
 * Revision 1.44  2012/11/08 22:11:33  mju
 * Add platform_win8.
 *
 * Revision 1.43  2012/08/29 14:45:18  mju
 * Add runmode and get/setRunMode.
 *
 * Revision 1.42  2011/10/19 19:02:23  mju
 * Remove redundant hdr.
 *
 * Revision 1.40  2010/09/28 13:49:20  mju
 * Add initialize/uninitialize.
 *
 * Revision 1.39  2010/03/04 23:58:22  bcolombe
 * Added KillProcess
 *
 * Revision 1.38  2010/02/11 18:06:33  mju
 * Add disable_suspend nested class.
 *
 * Revision 1.37  2009/08/11 16:30:55  mju
 * Add platform defns for win7 and winServer2008.
 *
 * Revision 1.36  2009/05/13 14:42:27  mju
 * Add disableCrashWindow.
 *
 * Revision 1.35  2009/02/19 21:23:32  bcolombe
 * Added EXECUTEFLAG_NoWaitForInit
 *
 * Revision 1.34  2008/05/23 14:59:16  mju
 * Add isMainThread and setAsMainThread.
 *
 * Revision 1.33  2007/10/17 19:56:59  scowan
 * Changed return value of getCurrentThreadID to THREAD_ID.Changed return value of getCurrentThreadID to THREAD_ID.
 *
 * Revision 1.32  2007/10/17 19:40:15  scowan
 * Adjusted for the different unix factions.
 *
 * Revision 1.31  2007/10/17 19:28:48  scowan
 * Added method to get current thread ID.
 *
 * Revision 1.30  2007/05/22 21:23:58  dwilliss
 * Added GetProcessID()
 *
 * Revision 1.29  2007/02/14 14:45:23  mju
 * Add platform_WinVista.
 *
 * Revision 1.28  2006/08/16 21:23:34  mju
 * Add comment for getNumLogicalProcessors.
 *
 * Revision 1.27  2006/08/09 16:34:39  mju
 * Add get/setUserThreadLimit.
 *
 * Revision 1.26  2006/05/08 14:49:47  scowan
 * Added executeProcess method.
 *
 * Revision 1.25  2005/10/13 15:25:29  scowan
 * Made method static.
 *
 * Revision 1.24  2005/10/13 15:07:53  scowan
 * Added methods to deal with language selections.
 *
 * Revision 1.23  2005/10/12 22:25:26  scowan
 * Added multiple langiage methods.
 *
 * Revision 1.22  2005/09/22 14:17:35  drb
 * Added GetLocale method
 *
 * Revision 1.21  2005/08/29 16:32:36  mju
 * Add setEnvironmentValue.
 *
 * Revision 1.20  2005/06/08 15:14:07  mju
 * Add getHostDetails.
 *
 * Revision 1.19  2005/03/02 20:39:54  mju
 * Add setSuspendDisabled.
 *
 * Revision 1.18  2005/01/11 00:23:58  scowan
 * Added get computer name..
 *
 * Revision 1.17  2004/12/29 16:16:17  scowan
 * Added methods to determine the number of logical and physical processors in the system.
 *
 * Revision 1.16  2004/07/09 17:30:28  dwilliss
 * Needed to forward declare FILEPATH
 *
 * Revision 1.15  2004/07/09 17:27:40  dwilliss
 * Added SetCurrentWorkingDirectory
 *
 * Revision 1.14  2004/04/13 17:23:34  mju
 * Change Win32_Old to Win32_PreNT.
 *
 * Revision 1.13  2004/04/13 16:49:41  dwilliss
 * Fixed platform enums so Win32_NT did not include the Win32_Old bit
 *
 * Revision 1.12  2003/12/22 21:01:06  dwilliss
 * Added static methods to get/set main app's HINSTANCE (Windows only)
 *
 * Revision 1.11  2003/10/09 13:37:20  mju
 * Add GetTimerValue.
 *
 * Revision 1.10  2003/10/08 14:53:17  mju
 * Add GetTicksPerSecond.
 *
 * Revision 1.9  2003/09/15 15:39:03  dwilliss
 * Mismatched #ifdef from doxygen cleanup
 *
 * Revision 1.8  2003/09/15 13:49:56  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.7  2003/07/09 13:59:56  mju
 * nc.
 *
 * Revision 1.6  2003/07/08 17:27:54  scowan
 * Undefine class lib export at the end of the module.
 *
 * Revision 1.5  2003/07/03 20:03:21  scowan
 * Moved get tick count to here.
 *
 * Revision 1.4  2003/07/03 16:36:12  scowan
 * Added translate system error method.
 *
 * Revision 1.3  2003/05/20 20:45:50  mju
 * Add methods to get total/available physical/virtual memory.
 *
 * Revision 1.2  2003/05/20 18:00:57  mju
 * Add PLATFORM enum and GetPlatform.
 * Add WindowsIsUnicode.
 * Add Supports64BitFileSystem.
 * Add Sleep.
 *
 * Revision 1.1  2003/05/19 15:38:11  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_MISYSTEM_H
#define	INC_MI32_MISYSTEM_H

#ifndef	INC_MI32_MISTRING_H
	#include <mi32/mistring.h>
#endif

#ifndef	INC_MI32_DELEGATE_H
	#include <mi32/delegate.h>
#endif

#ifdef MISYSTEMDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif

class FILEPATH;

//! Container for static methods to access various system and execution environment valuses.
class CLASSLIBEXPORT MISYSTEM {
	public:
		typedef UINT64 THREAD_ID;
	
		enum EXECUTEFLAGS {
			EXECUTEFLAG_None = 0x0000,
			EXECUTEFLAG_WaitUntilFinished = 0x0001,		//!< Block this thread's execution until the generated process is done.
			EXECUTEFLAG_ShowConsole = 0x0002,					//!< Show console on process creation, available for Windows only
			EXECUTEFLAG_NoWaitForInit = 0x0004			//!< Don't call WaitForInputIdle to wait for process to finish initialization, Windows only
			};

		enum PLATFORM {
			PLATFORM_Unknown =			0,

			PLATFORM_Win32_PreNT =		0x000100,
			PLATFORM_Win95 =				(PLATFORM_Win32_PreNT | 0x01),		//! Microsoft Windows 95
			PLATFORM_Win98 =				(PLATFORM_Win32_PreNT | 0x02),		//! Microsoft Windows 98
			PLATFORM_WinME =				(PLATFORM_Win32_PreNT | 0x03),		//! Microsoft Windows Me

			PLATFORM_Win32_NT =				0x001000,
			PLATFORM_WinNT =					(PLATFORM_Win32_NT | 0x01),		//! Microsoft Windows NT 4
			PLATFORM_Win2000 =				(PLATFORM_Win32_NT | 0x02),		//! Microsoft Windows 2000
			PLATFORM_WinXP =					(PLATFORM_Win32_NT | 0x03),		//! Microsoft Windows XP
			PLATFORM_WinServer2003 =		(PLATFORM_Win32_NT | 0x04),		//! Microsoft Windows Server 2003
			PLATFORM_WinVista =				(PLATFORM_Win32_NT | 0x05),		//! Microsoft Windows Vista
			PLATFORM_WinServer2008 =		(PLATFORM_Win32_NT | 0x06),		//! Microsoft Windows Server 2008
			PLATFORM_Win7 =					(PLATFORM_Win32_NT | 0x07),		//! Microsoft Windows 7
			PLATFORM_WinServer2008_R2 =	(PLATFORM_Win32_NT | 0x08),		//! Microsoft Windows Server 2008 R2
			PLATFORM_Win8 =					(PLATFORM_Win32_NT | 0x09),		//! Microsoft Windows 8

			PLATFORM_Unix =				0x002000,
			PLATFORM_MacOSX =				(PLATFORM_Unix | 0x01),
			PLATFORM_LinuxX86 =			(PLATFORM_Unix | 0x02),
			PLATFORM_UnixSun =			(PLATFORM_Unix | 0x03),
			PLATFORM_UnixSGI =			(PLATFORM_Unix | 0x04),
			PLATFORM_UnixIBM =			(PLATFORM_Unix | 0x05),
			};
			
		enum PROCESSRESULT {
			PROCESSRESULT_Finished = 0,
			PROCESSRESULT_ExitError,
			PROCESSRESULT_ExternalStop,
			PROCESSRESULT_DiedCoreDump,
			PROCESSRESULT_DiedNoCoreDump
			};

		//! Process run mode.
		enum RUNMODE {
			RUNMODE_User,			//!< User (interactive) process (default if not set)
			RUNMODE_Job				//!< Job (batch) process
			};

		//! Sentry class to enable/disable system suspend during long non-interactive operations.
		class DISABLE_SUSPEND {
			public:
				//! Constructor, disables suspend.
				DISABLE_SUSPEND ()
					{ MISYSTEM::SetSuspendDisabled(true); }
				//! Destructor, enables suspend.
				~DISABLE_SUSPEND ()
					{ MISYSTEM::SetSuspendDisabled(false); }
			};

		//! Disable dialog that may be displayed when process crashes.
		//! Use with care, as process will simply terminate on crash.
		static void DisableCrashWindow ();

		//! Execute a process with possible additional parameters.
		//! Double quotes are required for the process string and any parameters that have spaces.
		//! If called on UNIX systems with EXECUTEFLAG_WaitUntilFinished specified, only one delegate per thread can be assigned.
		//! If another delegate is assigned for the same thread, the other is dropped.  Windows systems do not have this limitation.
		static ERRVALUE ExecuteProcess (
			const MISTRING& Process,								//!< Process to execute plus any parameters.  If process is not an absolute path, OS process location rules apply.
			EXECUTEFLAGS ExeFlags = EXECUTEFLAG_None,
			FastDelegate<void(PTRUINT, PROCESSRESULT)> DelegateOnProcessEnd = 0,		//!< Called when the process is finished
			PTRUINT* ProcessID = 0									//!< Process ID of the child created RETURNED, if not NULL and EXECUTEFLAG_WaitUntilFinished is not specified and DelegateOnProcessEnd is assigned
			);

		//! Get the current language set for the calling thread.
		//! Depending on the situation, there may be more than one language selected in the order of first language prefered.
		//! The default language set order is:
		//! 1.  The code stored in the user's tntproc.ini file, if the entry exists.
		//! 2.  The code returned by GetSystemLanguage().
		//! 2.  English code "enu".
		//! @return ISO 639-2 three character language codes or ISO 639-1 two character language codes with possible ISO 3166 country codes
		static const MISTRINGLIST& GetCurrentLanguageSet ();
			
		//! Get current user name.
		//! @return MISTRING containing name, will be 'UnknownUser' if cannot determine name.
		static MISTRING GetCurrentUserName ();

		//! Get current thread ID
		//! @return The ID of the thread that calls this method
		static THREAD_ID GetCurrentThreadID ();
	
		//! Get value of named environment variable.
		//! @return MISTRING containing value, will be empty if no matching environment variable.
		static MISTRING GetEnvironmentValue (
			const char *name								//!< Environment variable name
			);

		//! Get host computer details.
		//! Returns multi-line string which may include OS name, version, service pack, processor,
		//! total physical and virtual memory, and other non-user-specific information.
		static MISTRING GetHostDetails ();
		
		//! Get host computer name
		//! @return MISTRING containing computer (non DNS) name, will be 'LocalHost' if cannot determine name.
		static MISTRING GetHostName ();

		//! Get the number of logical processors
		//! @return the number of logical processors in the system, this does include hyperthreaded processors
		//! Return value will always be at least the value returned by GetNumPhysicalProcessors.
		//! Will return one if the number of processors cannot be determined.
		static int GetNumLogicalProcessors ();

		//! Get the number of physical processors
		//! @return the number of actual physical processors in the system, this does not include hyperthreaded processors
		//! Will return zero if the number of physical processors cannot be determined.
		static int GetNumPhysicalProcessors ();

		//! Get amount of available physical memory.
		//! @return Number of bytes, -1 if unsupported, < -1 if error.
		static INT64 GetPhysicalMemoryAvailable ();

		//! Get amount of total physical memory.
		//! @return Number of bytes, -1 if unsupported, < -1 if error.
		static INT64 GetPhysicalMemoryTotal ();

		//! Get platform.
		static PLATFORM GetPlatform ();

		//! Get the numeric ID of the current process
		static UINT32 GetProcessID ();

		//! Get process run mode.
		static RUNMODE GetRunMode ();

		//! Get the system encoding that is used in filenames and other system strings
		//! For Windows and MAC, they are static, for LINUX and SUN, it depends on the current locale setting
		//! @return The system encoding
		static CHAR_ENCODING GetSystemEncoding ();
			
		//! Get the language setup for the system (set via the current user)
		//! @return The ISO639-1 two character language code for the current user with the ISO3166 county/region two character name.  (Eg:  en-US)
		static const char* GetSystemLanguage ();
			
		//! Returns a simple clock tick of unknown starting time. 
		static UINT32 GetTickCount ();

		//! Get number of clock ticks per second.
		static UINT32 GetTicksPerSecond ();

		//! Return 'timer' value.
		//! Precision of timer is system-dependent, ranging from 1ms and 100ns.
		//! @return Number of seconds from arbitrary starting time.
		static double GetTimerValue ();

		//! Get user preference for thread limit.
		//! If no thread limit set, return value is based on number of processor cores, otherwise
		//! will return smalller of user limit and number of cores.
		static int GetUserThreadLimit (
			bool IncludeLogicalCores				//!< Include logical (hyperthreading) cores if any
			);

		//! Get amount of available virtual memory.
		//! The return value may change if the OS supports dynamic page/swap file allocation.
		//! @return Number of bytes, -1 if unsupported, < -1 if error.
		static INT64 GetVirtualMemoryAvailable ();

		//! Get total amount of virtual memory.
		//! The return value may change if the OS supports dynamic page/swap file allocation.
		//! @return Number of bytes, -1 if unsupported, < -1 if error.
		static INT64 GetVirtualMemoryTotal ();

		//! Initialize system.
		static void Initialize ();

		//! Determine if current thread is set as "main" thread.
		//! Typically the "user interface" is run in the main thread.
		static bool IsMainThread ();

		//! Kill process by pid
		static void KillProcess (int ProcessID);

		//! Given an ISO639-2 three character language code, return the equivalent ISO639-1 two character language code
		//! @return The ISO639-1 two character language code or NULL if one is not found
		static const char* LookupISO639_1Code (
			const char* iso639_2code				//!< ISO639-2 three character language code
			);
			
		//! Given an ISO639-1 two character language code, return the equivalent ISO639-2 three character language code
		//! The returned ISO639-2 code may have two sets of codes separated by a '/'.
		//! @return The ISO639-2 three character language code or NULL if one is not found
		static const char* LookupISO639_2Code (
			const char* iso639_1code				//!< ISO639-1 two character language code
			);

		//! Set current thread as "main thread".
		static void SetAsMainThread ();
			
		//! Set the current language set for the calling thread.
		//! If there is more than one language listed, then the order is of first language prefered.
		static void SetCurrentLanguageSet (
			const MISTRINGLIST& iso639				//!< ISO 639-2 three character language code or ISO 639-1 two character language code with a possible ISO 3166 country code list
			);
			
		//! Set the current working directory.
		//! Note, to get the current working directory, construct a FILEPATH
		//! with FILEPATH::PATHTYPE_Current.
		static ERRVALUE SetCurrentWorkingDirectory (
			const FILEPATH& filepath
			);

		//! Set value of named environment variable (char* value).
		static void SetEnvironmentValue (
			const char *name,								//!< Environment variable name
			const char *value								//!< Value to set
			);

		//! Set process "run" mode.
		static void SetRunMode (
			RUNMODE runmode								//!< Run mode
			);

		//! Enable or prevent system processor from entering suspended / sleeping state.
		//! Keeps counter so each time is disabled must be matched by call to enable.
		//! Initial state is to enable suspend.
		//! May not be supported on all platforms.
		static void SetSuspendDisabled (
			bool disable									//!< True to disable suspend, false to enable
			);

		//! Set user preference for thread limit.
		//! Intended to allow user to limit number of threads below default for evaluating performance.
		static void SetUserThreadLimit (
			int MaxThreads									//!< Maximum threads to return from GetUserThreadLimit
			);

		//! Delay for specified time.
		static void Sleep (
			double delay									//!< Time to delay in seconds
			);

		//! Determine if system supports 64-bit filesystems.
		static bool Supports64BitFileSystem ();

		//! Translate OS error code to MicroImages error code
		//! Does NOT do a SetErrPosn() on the error code, that is up to the caller
		static ERRVALUE TranslateSystemError (
			ERRVALUE DftErrCode,					//!< Default error code to use if system error not specifically handled
			INT32& SystemError					//!< Operating sytem error RETURNED
			);

	#ifdef WIN32
		//! Determine if Windows supports Unicode (Windows only).
		//! This may be used to determine if the 'wide character' versions of APIs may be used.
		//! @return true if supported, false if not.
		static bool WindowsIsUnicode ();


		//! Get the HINSTANCE of the application (Windows only)
		//! Sorry, you'll have to cast the return value to an HINSTANCE.
		//! this include file does not define any Windows types, nor should it.
		static void* GetMainHINSTANCE();

		//! Set the HINSTANCE of the application (Windows only)
		static void SetMainHINSTANCE(void*);
	#endif

		//! Uninitialize system.
		static void Uninitialize ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		MISYSTEM ();				//! Unimplemented
		#endif //!< GENERATING_DOXYGEN_OUTPUT
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(MISYSTEM::EXECUTEFLAGS)
#endif

#undef	CLASSLIBEXPORT

#endif	// INC_MI32_MISYSTEM_H
