/**
 * \file movie.h <mi32/movie.h>
 * \brief Header for MicroImages Movie functions
 *
 * \if NODOC
 * $Id: movie.h_v 1.8 2006/07/14 16:50:21 dwilliss Exp $
 *
 * $Log: movie.h_v $
 * Revision 1.8  2006/07/14 16:50:21  dwilliss
 * got rid of sml structure that was only used in the implementation module
 *
 * Revision 1.7  2005/12/19 16:56:49  mju
 * Declare filepath.
 *
 * Revision 1.6  2005/12/19 16:24:26  mju
 * Use filepath.
 *
 * Revision 1.5  2005/03/31 16:57:18  fileserver!dwilliss
 * Rename one of our types to MIUNICODE because it conflicted with a Microsoft #define
 *
 * Revision 1.4  2003/09/15 13:49:56  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.2  2000/06/22 21:23:42  mju
 * Include stddefns so compiles in isloation.
 *
 * Revision 1.1  1999/03/31 16:43:49  vdronov
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_MOVIE_H
#define	INC_MI32_MOVIE_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

class FILEPATH;

/* frame rate */
#define MOVIE_FRAMERATE_23_976		1	//!<  23.976 (24000/1001) fps - NTSC encapsulated film rate 
#define MOVIE_FRAMERATE_24			2	//!<  24 fps - Standard international cinema film rate 
#define MOVIE_FRAMERATE_25			3	//!<  25 fps - PAL (625/50) video frame rate 
#define MOVIE_FRAMERATE_29_970		4	//!<  29.97 (30000/1001) fps - NTSC video frame rate 
#define MOVIE_FRAMERATE_30			5	//!<  30 fps - NTSC drop-frame (525/60) video frame rate 
#define MOVIE_FRAMERATE_50			6	//!<  50 fps - Double frame rate / progressive PAL 
#define MOVIE_FRAMERATE_59_940		7	//!<  59.94 (60000/1001) fps - Double frame rate NTSC 
#define MOVIE_FRAMERATE_60			8	//!<  60 fps - Double frame rate drop-frame NTSC 


/*----------------------------------------------------------------------------*/
/*		Macros and functions prototypes														*/
/*----------------------------------------------------------------------------*/

#if defined(__cplusplus)
extern "C" {
#endif

int	MmovieEncodeInit (void**);
int	MmovieEncodeStart (void*, const FILEPATH&);
int	MmovieEncodeFrame (void*, void*);
void	MmovieEncodeStop (void*);
void	MmovieEncodeExit (void**);

int	MmovieEncodeSetFrameWidth (void*, INT32);
int	MmovieEncodeSetFrameHeight (void*, INT32);
int	MmovieEncodeSetFrameRate (void*, UINT8);
Widget	MmovieEncodeCreateParmForm (void*, Widget);
int	MmovieEncodeGetExt (void*, char*, int);


#if defined(__cplusplus)
}
#endif

#endif	//!<  ifndef INC_MI32_MOVIE_H 
