/**
 * \file mi32/textresitem.h
 * \brief TEXTRESITEM class definition
 *
 * \if NODOC
 * $Id: textresitem.h_v 1.1 2006/10/06 17:39:48 mju Exp $
 *
 * $Log: textresitem.h_v $
 * Revision 1.1  2006/10/06 17:39:48  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_TEXTRESITEM_H
#define	INC_MI32_TEXTRESITEM_H

#ifndef	INC_MI32_TEXTRES_H
#include <mi32/textres.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class TEXTRESITEM_PRIV;
#endif // GENERATING_DOXYGEN_OUTPUT

#ifdef MISYSTEMDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif

//! Text resource item container.
//! Used to encapsulate various ways to reference a text resource.
//! May contain an actual string instance or a reference to a string location in the resource file.
class CLASSLIBEXPORT TEXTRESITEM {
	public:

		//! Default constructor.
		TEXTRESITEM (
			);

		//! Copy constructor.
		TEXTRESITEM (
			const TEXTRESITEM& rhs
			);

		//! Construct from item name.
		TEXTRESITEM (
			const char *name,
			TEXTRES_NAMESET nameset = TEXTRES_NAMESET_General
			);

		//! Construct from TEXTID.
		TEXTRESITEM (
			TEXTID textid
			);

		//! Construct from MISTRING.
		TEXTRESITEM (
			const MISTRING& rhs
			);

		//! Destructor.
		~TEXTRESITEM ();

		//! Assignment.
		TEXTRESITEM& operator= (
			const TEXTRESITEM& rhs
			);

		//! Assign from item name in 'general' nameset.
		TEXTRESITEM& operator= (
			const char *name
			);

		//! Assign from TEXTID.
		TEXTRESITEM& operator= (
			TEXTID rhs
			);

		//! Assign from MISTRING.
		TEXTRESITEM& operator= (
			const MISTRING& rhs
			);

		#ifndef GENERATING_DOXYGEN_OUTPUT
		// Used by MILIST::Sort().
		bool operator< (
			const TEXTRESITEM& rhs
			) const;
		#endif // GENERATING_DOXYGEN_OUTPUT

		//! Assign from item name in specified nameset.
		//! @return true if item exists, false if not.
		bool Assign (
			const char *name,
			TEXTRES_NAMESET nameset = TEXTRES_NAMESET_General
			);

		//! Assign from item name in specified nameset.
		//! @return true if item exists, false if not.
		bool Assign (
			const UINT8 *name,				//! Name, assumed in UTF8 encoding for NAMESET_NoLookup
			TEXTRES_NAMESET nameset = TEXTRES_NAMESET_General
			);

		//! Clear item.
		void Clear (
			);

	#ifndef NODLLLINKAGE
		//! Create identifier from XML node.
		//! Node must be in one of the following forms:
		//!
		//!		\<nodename\>text_reference\</nodename\>
		//!		\<nodename lookup="yes"\>text_reference\</nodename\>
		//!		\<nodename lookup="no"\>text string\</nodename\>
		//!
		//! @return true if successful, false if not
		bool CreateFromXML (
			const XMLNODE *itemnode
			);
	#endif

		//! Get resource string.
		const MISTRING& GetString (
			) const;

		#ifndef GENERATING_DOXYGEN_OUTPUT
		// Used by equality/inequality.
		bool IsEqual (
			const TEXTRESITEM& rhs
			) const;
		#endif // GENERATING_DOXYGEN_OUTPUT

		//! Determine if item references valid text resource or string.
		bool IsValid (
			) const;

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		TEXTRESITEM_PRIV *m_pPriv;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//! Equality.
inline bool operator== (
	const TEXTRESITEM& lhs,
	const TEXTRESITEM& rhs
	) {
	return (lhs.IsEqual(rhs));
	}

//! Inequality.
inline bool operator!= (
	const TEXTRESITEM& lhs,
	const TEXTRESITEM& rhs
	) {
	return (!lhs.IsEqual(rhs));
	}



#undef	CLASSLIBEXPORT

#endif	// INC_MI32_TEXTRESITEM_H
