/**
 * \file rvc/dispparm.h 
 * \brief interface for the RVCDISPPARM class.
 *
 * \if NODOC
 * $Id: dispparm.h_v 1.8 2006/09/08 19:28:14 scowan Exp $
 *
 * $Log: dispparm.h_v $
 * Revision 1.8  2006/09/08 19:28:14  scowan
 * Change enum from serialize to its own enum.
 *
 * Revision 1.7  2005/06/30 21:01:58  scowan
 * Added makeparms.
 *
 * Revision 1.6  2003/10/07 17:34:19  scowan
 * Object virtual method changes.
 *
 * Revision 1.5  2003/09/30 16:52:06  dwilliss
 * Doxygen
 *
 * Revision 1.4  2003/09/25 21:20:04  dwilliss
 * Fixed doxygen comment
 *
 * Revision 1.3  2003/09/15 13:49:45  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.2  2001/06/15 17:39:45  scowan
 * Added to RVC namespace.
 *
 * Revision 1.1  2000/11/29 17:14:23  scowan
 * Initial revision
 *
 * \endif
**/

//!
//!	\class RVC::DISPPARM rvc/dispparm.h
//!	\brief Manages RVC::OBJTYPE_DispParm objects.
//!
//!	RVC Object Types that this class supports: RVC::OBJTYPE_DispParm
//!
//!	Default RVC Object Type for this class: RVC::OBJTYPE_DispParm
//!
//!	Valid parents for this class:
//!		- RVC::OBJTYPE_File
//!		- RVC::OBJTYPE_Folder
//!		- RVC::OBJTYPE_Database
//!		- RVC::OBJTYPE_Vector
//!		- RVC::OBJTYPE_Raster
//!		- RVC::OBJTYPE_CAD
//!		- RVC::OBJTYPE_TIN
//!		- RVC::OBJTYPE_Hyperspectral
//!		- RVC::OBJTYPE_RasterSet
//!		- RVC::OBJTYPE_Database_Raster
//!		- RVC::OBJTYPE_Database_VectorPoint
//!		- RVC::OBJTYPE_Database_VectorLine
//!		- RVC::OBJTYPE_Database_VectorPolygon
//!		- RVC::OBJTYPE_Database_CAD
//!		- RVC::OBJTYPE_Database_TIN

#ifndef INC_RVC_DISPPARM_H
#define INC_RVC_DISPPARM_H

#ifndef INC_RVC_SERIALZE_H
	#include <rvc/serialze.h>
#endif

namespace RVC {

class DISPPARM : public RVC::SERIALIZE {
	public:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class MAKEPARMS;
		#endif

		//! Default constructor	
		DISPPARM (
			);

		//! Copy constructor
		DISPPARM (
			const RVC::DISPPARM& obj
			);

		//! Destructor
		virtual ~DISPPARM (
			);

		//! Assignment
		RVC::DISPPARM& operator= (
			const RVC::DISPPARM& rhs
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		//! Implementation of RVC::OBJECT virtual methods
		virtual OBJTYPE v_GetDftObjectType () const;
		#endif // GENERATING_DOXYGEN_OUTPUT

	};


class DISPPARM::MAKEPARMS : public SERIALIZE::MAKEPARMS {
	public:
		MAKEPARMS (
			UINT16 Version = 1,				 			//!< Version number to assign to new object
			OBJTYPE ForObjType = OBJTYPE_All,		//!< RVC object that this serialize object is referenced by
			DISPPARMUSAGE Usage = DISPPARMUSAGE_Display			//!< Usage of this object
			) : SERIALIZE::MAKEPARMS(RVC::OBJTYPE_DispParm, Version, ForObjType, Usage)
			{}
	};			

}		//! End of RVC namespace

#endif 	//!< INC_RVC_DISPPARM_H
