/**
 * multrast.h - MULTIRASTER class declaration
 *
 * $Id: multrast.h_v 1.26 2008/02/28 20:42:23 mju Exp $
 *
 * $Log: multrast.h_v $
 * Revision 1.26  2008/02/28 20:42:23  mju
 * Add makePyramidSetNextTiers.
 *
 * Revision 1.25  2007/10/30 22:05:42  mju
 * Don't  use const for objitem in make so can return new location.
 *
 * Revision 1.24  2007/07/03 20:29:02  mju
 * Remove updatePyramid as not implemented or used.
 *
 * Revision 1.23  2007/06/21 14:48:34  mju
 * Change makeparms to eliminate unused and redundant values and require raster size.
 * Don't inline ctors/dtor.
 *
 * Revision 1.22  2007/06/18 19:22:40  mju
 * Exclude private sections from doxygen.
 *
 * Revision 1.21  2007/05/24 22:26:19  dfriberg
 * Add pyramid flags
 *
 * Revision 1.20  2007/05/23 20:49:49  jjohnson
 * Add optional band parameter to makeparms
 *
 * Revision 1.19  2007/01/29 18:51:12  dfriberg
 * Typo in docs
 *
 * Revision 1.18  2007/01/29 18:40:14  dfriberg
 * Documentation
 *
 * Revision 1.17  2007/01/12 22:15:48  dfriberg
 * API changes
 *
 * Revision 1.16  2006/12/13 23:02:43  dfriberg
 * Explicitly set base class in copy constructor
 *
 * Revision 1.15  2006/12/13 21:27:46  dfriberg
 * Add Make method that takes objitem
 *
 * Revision 1.14  2006/10/16 15:04:56  dfriberg
 * Don't inline SetLinkParms method
 *
 * Revision 1.13  2006/10/13 01:35:51  dfriberg
 * Remove unneeded methods, and add temp file member
 *
 * Revision 1.12  2006/10/07 01:12:56  dfriberg
 * Moved some code to RVCSYS
 *
 * Revision 1.11  2006/09/22 00:56:05  dfriberg
 * Now can just add link without worrying which tile it belongs in
 *
 * Revision 1.10  2006/09/15 14:06:23  scowan
 * Fixed include file issues.
 *
 * Revision 1.9  2006/09/14 16:07:26  dfriberg
 * Rename method
 *
 * Revision 1.8  2006/09/07 15:26:14  dfriberg
 * Redesigned multiraster
 *
 * Revision 1.7  2006/03/24 18:39:51  dfriberg
 * Add method to get linked file for grid tiles
 *
 * Revision 1.6  2006/03/22 16:09:00  dfriberg
 * Compression parameters added to MakePyramidSet
 *
 * Revision 1.5  2006/03/21 00:56:10  dfriberg
 * Doxygen
 *
 * Revision 1.4  2006/03/13 17:57:39  dfriberg
 * MULTIRASTER parameter passed to Make no longer const
 *
 * Revision 1.3  2006/03/08 05:45:31  dfriberg
 * Add nullmask multiraster make method
 *
 * Revision 1.2  2006/02/22 23:35:26  dfriberg
 * Add RASTER::USAGE member to MULTIRASTER::MAKEPARMS
 *
 * Revision 1.1  2006/02/16 02:10:33  dfriberg
 * Initial revision
 *
 **/

#ifndef INC_RVC_MULTRAST_H
#define INC_RVC_MULTRAST_H

#ifndef INC_RVC_RASTER_H
	#include <rvc/raster.h>
#endif

#include <vector>

#define RVC_DEFAULT_CREDENTIALS = 0

namespace RVC {

class CREDENTIALS;

class MULTIRASTER : public RASTER {
	public:

		enum TYPE {
			TYPE_None			= 0,
			TYPE_DirectLink,
			TYPE_ObjectPath,
			TYPE_LegacyInode
			};

		class MAKEPARMS {
			public:

				//! Construct with required parameters
				MAKEPARMS (
					INT32 TotalLins,						//!< Total number of lines in raster
					INT32 TotalCols,						//!< Total number of columns in raster
					INT32 LinsPerTile,					//!< Number of lines per linked raster tile
					INT32 ColsPerTile,					//!< Number of columns per linked raster tile
					RVC::IMAGE::CELLTYPE CellType,	//!< Celltype of raster
					TYPE Type,								//!< Type of multiraster to create
					RASTER::LINKTYPE m_LinkType = RASTER::LINKTYPE_NoLink	//!< Link type, required for TYPE_DirectLink
					);

				~MAKEPARMS () { }

				//! Set band number for multi-band link.
				void SetBandNum (
					INT32 BandNum
					) { m_Band = BandNum; }

				void SetUsage (
					RASTER::USAGE usage
					) { m_Usage = usage; }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				friend class MULTIRASTER;

				INT32 m_TotalLins;
				INT32 m_TotalCols;
				INT32 m_LinsPerTile;
				INT32 m_ColsPerTile;
				IMAGE::CELLTYPE m_CellType;
				MULTIRASTER::TYPE m_Type;
				RASTER::LINKTYPE m_LinkType;
				RASTER::USAGE m_Usage;
				INT32 m_Band;
				#endif
			};

		MULTIRASTER (
			);

		MULTIRASTER (
			const MULTIRASTER& rhs
			);

		virtual ~MULTIRASTER (
			);

		//! Add linked raster to position at specified location.  The DestRect must be equal to the size of the linked raster.
		ERRVALUE AddLink (
			const OBJITEM& LinkRastObj,
			const LRECT2D& DestRect,
			bool RebuildPyramids
			);

		//! Add link to image file at specified location.
		//! The DestRect must be equal to the size of the linked raster.
		ERRVALUE AddLink (
			const RASTER::LINKPARMS& parms,
			const LRECT2D& DestRect				//Dimensions of linkable raster assumed to be same as DestRect.
			);

		//! Clear everything from tile specified.
		ERRVALUE ClearTile (
			const LPOINT2D& TilePos
			);

		ERRVALUE GetGridLinkFile (
			FILEPATH& linkfile,
			INT32 GridLine,
			INT32 GridCol
			);

		//! Get the OBJITEM for the raster at the specified grid location.
		ERRVALUE GetGridLinkItem (
			OBJITEM& obj,
			INT32 GridLine,
			INT32 GridCol
			);

		INT32 GetGridTileLines (
			) const;

		INT32 GetGridTileColumns (
			) const;

		ERRVALUE GetLinkCommonPath (
			FILEPATH& filepath
			);

		ERRVALUE GetTileLinkPaths (
			std::vector<FILEPATH>& LinkFiles,
			const LPOINT2D& TilePos
			);

		//! Make a new MULTIRASTER
		ERRVALUE Make (
			const OBJECT& parent,
			const DESCRIPTOR& desc,
			MAKEPARMS& parms,
			const CREDENTIALS* Credentials = 0
			);

		//! Make a new MULTIRASTER
		ERRVALUE Make (
			OBJITEM& objitem,				//!< Object location passed/returned
			MAKEPARMS& parms,
			const CREDENTIALS* Credentials = 0
			);

		//! Make a MULTIRASTER to serve as the NULLMASK.  Will also create
		//! MULTIRASTER pyramid tiers (and RASTER pyramid tiers if needed).
		ERRVALUE MakeNullMask (
			const CREDENTIALS* Credentials = 0
			);

		//! Make pyramid tiers.  Will first make a multiraster pyramid tier for each item in
		//! samplerates at specified rate.  Subsequently will make ordinary raster pyramid tiers
		//! if necessary.
		//!
		//! Assumes samplerates is sorted, and will only make ordinary raster pyramid tiers of lower
		//! resolution than given in this array.
		//!
		//! Note: you must not specify a lower resolution than what is available in the reduced resolution
		//! data for ANY image in the Multiraster.  To get a list of valid sample rates, call
		//! RVC::RASTER::GetEmbeddedPyramidList on the multiraster before using this method.
		ERRVALUE MakePyramidSet (
			RASTER::COMPTYPE comptype,
			UINT16 compquality,
			const SIMPLE_ARRAY<INT32>& samplerates,
			IMAGE::PYRAMID flags = IMAGE::PYRAMID_Default,
			const CREDENTIALS* Credentials = 0
			);

		//! Make next pyramid tiers as a tileset.
		//! \internal Used when internal pyramid tiers would be too large.
		static ERRVALUE MakePyramidSetNextTiers (
			const RVC::OBJITEM& ObjItemPrevTier,
			RASTER::COMPTYPE CompressType,				//!< Must be COMPTYPE_JP2 at present
			UINT16 CompressRatio,
			const CREDENTIALS* Credentials
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		virtual ERRVALUE v_OpenObject (OPENMODE OpenMode, MDLGPARENT parent, const CREDENTIALS* Credentials);

		MAKEPARMS m_MakeParmsMult;
		OBJECT m_TempFile;
		#endif
	};
} //End RVC namespace

#endif

