/**
 * \file rvc/script.h 
 * \brief interface for the RVC::SCRIPT class.
 *
 * \if NODOC
 * $Id: script.h_v 1.9 2006/06/27 15:04:48 scowan Exp $
 *
 * $Log: script.h_v $
 * Revision 1.9  2006/06/27 15:04:48  scowan
 * Make method now fills in objitem with created object info.
 *
 * Revision 1.8  2005/07/01 21:06:26  scowan
 * Do not make dtor pure.
 *
 * Revision 1.7  2005/06/30 21:02:19  scowan
 * Added makeparms.
 *
 * Revision 1.6  2005/06/23 20:19:43  scowan
 * Changed format.
 *
 * Revision 1.5  2003/09/30 16:52:06  dwilliss
 * Doxygen
 *
 * Revision 1.4  2003/09/25 21:56:58  dwilliss
 * doxygen
 *
 * Revision 1.3  2003/09/15 13:49:45  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.2  2001/06/15 16:53:46  scowan
 * Added to RVC namespace.
 *
 * Revision 1.1  2000/11/29 16:03:56  scowan
 * Initial revision
 *
 * \endif
**/

//!	\class RVC::SCRIPT rvc/script.h
//!	\brief Interface for RVC objects that support SML query scripts
//!
//!	RVC Object Types that this class supports: Derived class
//!
//!	Default RVC Object Type for this class: Derived class
//!
//!	Valid parents for this class: Derived class

#ifndef INC_RVC_SCRIPT_H
#define INC_RVC_SCRIPT_H

#ifndef INC_RVC_TEXT_H
	#include <rvc/textobj.h>
#endif

namespace RVC {

class SCRIPT : public RVC::TEXTOBJ {
	public:
		enum USAGE {
			USAGE_Normal = 0,
			USAGE_ComputedField = 1
			};
			
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class MAKEPARMS;
		#endif
		
		//! Default constructor
		SCRIPT (
			);

		//! Copy constructor
		SCRIPT (
			const RVC::SCRIPT& rhs
			);

		//! Destructor
		virtual ~SCRIPT (
			);

		//! Assignment
		RVC::SCRIPT& operator= (
			const RVC::SCRIPT& rhs
			);

		//! Get the usage for this script
		//! @return USAGE for this script object
		USAGE GetUsage (
			) const;		
			
		//! Get the database table field number for this script
		//! @return Database table field number for this script object
		UINT32 GetFieldNumber (
			) const;		
			
		//! Get correct OBJTYPESET for all text script OBJTYPE's
		//! @return OBJTYPESET for all text script OBJTYPE's
		static RVC::OBJTYPESET GetObjTypeSet (
			);
			
		//! Determine if the object type is a script text object
		static bool IsScript (
			RVC::OBJTYPE ObjType
			);

		//! Create a new rvc script object
		ERRVALUE Make (
			const RVC::OBJECT& parent,				//!< Parent rvc object to create text object under
			const MAKEPARMS& MakeParms,
			const RVC::DESCRIPTOR& Descriptor	//!< Name of new rvc text object
			);

		//! Create a new rvc script object
		ERRVALUE Make (
	    	RVC::OBJITEM& ObjItem,					//!< Parent object and DESCRIPTOR of object to create, new location RETURNED
			const MAKEPARMS& MakeParms,
			const char* Source = 0					//!< Source of rvc text object creator, default is GetAppName()
			);

	};


class SCRIPT::MAKEPARMS : public TEXTOBJ::MAKEPARMS {
	public:
		// Default ctor
		MAKEPARMS (
			RVC::OBJTYPE ObjType,
			UINT32 FieldNum = 0,
			SCRIPT::USAGE Usage = SCRIPT::USAGE_Normal
			);
			
		UINT32 GetFieldNum (
			) const;
			
		SCRIPT::USAGE GetUsage (
			) const;
			
		void SetFieldNum (
			UINT32 FieldNum
			);
			
		void SetUsage (
			SCRIPT::USAGE Usage
			);
	};	
	
}		//! End of RVC namespace

#endif 	//!< INC_RVC_SCRIPT_H
