/** 
 * \file rvc/uieditobjectname.h 
 * \brief Definitions for RVC::UI::CTRL_EDIT_OBJECTNAME and RVC::UI::FORM_EDIT_OBJECTNAME classes.
 *
 * \if NODOC
 * $Id: uieditobjectname.h_v 1.3 2008/09/02 21:40:48 mju Exp $
 *
 * $Log: uieditobjectname.h_v $
 * Revision 1.3  2008/09/02 21:40:48  mju
 * Fix doxygen warning.
 *
 * Revision 1.2  2006/10/13 20:33:48  mju
 * Use rvc objectname hdr instead of rvcdefns.
 *
 * Revision 1.1  2006/06/26 20:03:30  mju
 * Initial revision
 *
 * Revision 1.7  2006/04/13 16:23:47  dwilliss
 * *** empty log message ***
 *
 * Revision 1.6  2005/06/24 20:28:36  mju
 * Include edit hdr
 *
 * Revision 1.5  2005/06/23 22:00:18  mju
 * Support delegates.
 * Remove template versions as no longer used.
 * Remove most methods from form, just use getEditCtrl to access.
 *
 * Revision 1.4  2005/03/31 16:57:08  fileserver!dwilliss
 * Rename one of our types to MIUNICODE because it conflicted with a Microsoft #define
 *
 * Revision 1.3  2003/12/11 15:07:06  mju
 * Remove unnecessary base virtual call for onchangevalue.
 *
 * Revision 1.2  2003/10/01 22:32:25  dwilliss
 * doxygen
 *
 * Revision 1.1  2003/09/29 22:59:26  scowan
 * Initial revision
 * \endif
**/

#ifndef	INC_RVC_UIEDITOBJECTNAME_H
#define	INC_RVC_UIEDITOBJECTNAME_H

#ifndef	INC_MGUI_EDIT_H
#include <mgui/edit.h>
#endif

#ifndef	INC_RVC_OBJECTNAME_H
#include <rvc/objectname.h>
#endif

namespace RVC {
namespace UI {

//===================================================================================================================
//! Object name 'edit' control.
class CTRL_EDIT_OBJECTNAME : public MGUI::CTRL_EDIT_BASE {
	public:

		//! Constructor.
		CTRL_EDIT_OBJECTNAME (
			);

		//! Destructor.
		virtual ~CTRL_EDIT_OBJECTNAME (
			);

		//! Clear to empty string.
		void ClearValue (
			bool notify = true							//!< Call OnChangeValue() if control has already been created
			);

		//! Create control.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedSize
			);

		//! Get current string value.
		const RVC::OBJECTNAME& GetValue (
			) const { return (m_value); }

		//! Read value from INI file.
		//! If Create() has already been performed the control's state will be updated.
		void IniRead (
			INIHANDLE IniHandle,						//!< Handle to INI file to read from, 0 for default
			const char *IniGroup,					//!< INI group to read from
			const char *IniField,					//!< INI field to read from
			bool notify = true						//!< Call OnChangeValue() if control has already been created
			);

		//! Write current value to INI file.
		void IniWrite (
			INIHANDLE IniHandle,						//!< Handle to INI file to read from, 0 for default
			const char *IniGroup,					//!< INI group to read from
			const char *IniField						//!< INI field to read from
			) const { ::IniWrite(IniHandle,IniGroup,IniField,m_value); }

		//! Set delegate to call to perform validation.
		void SetDelegateOnValidate (
			FastDelegate<bool(MISTRING&)> delegate
			) { m_DelegateOnValidate = delegate; }

		//! Set current string value with validation.
		//! OnValidate() will be called and if valid, the string will be updated.
		//! No comparison is made with the current string value.
		void SetValue (
			const RVC::OBJECTNAME& name,			//!< New string to set
			bool notify = true						//!< Call OnChangeValue() if string actually updated
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		FastDelegate<bool(MISTRING&)> m_DelegateOnValidate;
		RVC::OBJECTNAME m_value;

		bool OnValidate (MISTRING& string) const;

		virtual void v_OnUserEdit ();
		virtual void v_UpdateValue (MISTRING& string);

		CTRL_EDIT_OBJECTNAME (const CTRL_EDIT_OBJECTNAME&);
		CTRL_EDIT_OBJECTNAME& operator= (const CTRL_EDIT_OBJECTNAME&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


//===================================================================================================================
//! Object name edit form with label and edit field.
class FORM_EDIT_OBJECTNAME : public MGUI::FORM_COMPOSITE {
	public:
		//! Constructor.
		FORM_EDIT_OBJECTNAME (
			);

		//! Destructor.
		virtual ~FORM_EDIT_OBJECTNAME (
			);

		//! Create form with label from resource lookup.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			const MISTRING& label,						//!< Label
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedSize,
			MGUI::CTRL_LABEL::STYLE labelstyle = MGUI::CTRL_LABEL::STYLE_LeftNoWrap
			);

		//! Retrieve string "edit" control for form.
		//! Use to configure control, set delegate, etc.
		CTRL_EDIT_OBJECTNAME& GetEditCtrl (
			) { return (m_editctrl); }

		//! Retrieve label control for form.
		//! Usually the label control is only retrieved for alignment purposes.
		MGUI::CTRL_LABEL& GetLabel (
			) { return (m_label); }

		//! Get current string value.
		//! Equivalent to GetEditCtrl().GetValue().
		const RVC::OBJECTNAME& GetValue (
			) const { return (m_editctrl.GetValue()); }

		//! No comparison is made with the current string value.
		//! Equivalent to GetEditCtrl().SetValue(...).
		void SetValue (
			const RVC::OBJECTNAME& name,			//!< New string to set
			bool notify = true						//!< Call OnChangeValue() if string actually updated
			) { m_editctrl.SetValue(name, notify); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		MGUI::CTRL_LABEL m_label;
		CTRL_EDIT_OBJECTNAME m_editctrl;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================

}	// End UI namespace
}	// End RVC namespace

#endif		//	INC_RVC_UIEDITOBJECTNAME_H

