/**
 * \file rvc/uistyle.h
 * \brief RVC User Interface controls for working with styles.
 *
 * \if NODOC
 * $Id: uistyle.h_v 1.7 2011/12/28 14:18:58 mju Exp $
 *
 * $Log: uistyle.h_v $
 * Revision 1.7  2011/12/28 14:18:58  mju
 * Remove redundant hdr.
 *
 * Revision 1.3  2007/02/13 18:11:44  dwilliss
 * changed parameters to icon selection dialog
 *
 * Revision 1.1  2006/09/20 15:10:58  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_RVC_UISTYLE_H
#define	INC_RVC_UISTYLE_H

#ifndef	INC_MGUI_LISTITEM_H
#include <mgui/listitem.h>
#endif

#ifndef	INC_MGUI_FORMRADIOBUTTONS_H
#include <mgui/formradiobuttons.h>
#endif

#ifndef	INC_MGUI_EDIT_H
#include <mgui/edit.h>
#endif

#ifndef	INC_MGUI_IMAGELST_H
#include <mgui/imagelst.h>
#endif

#ifndef	INC_RVC_STYLETYPE_H
#include <rvc/styletype.h>
#endif

#ifndef	INC_MI32_ELEMTYPE_H
#include <mi32/elemtype.h>
#endif

#ifndef	INC_RVC_OBJECT_H
#include <rvc/object.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations.
class MISTRING;
struct POINTSTYLE;
struct LINESTYLE;
struct MULTISTYLE;
struct POLYSTYLE;
struct TEXTSTYLE;
namespace RVC {
	class OBJITEM;
	class STYLE;
	}
#endif


namespace RVC {
namespace UI {

//! Enumeration of flags for style editor forms and dialogs
enum EDITSTYLEFLAGS {
	EDITSTYLEFLAG_None =							0x00000000,	//!< No flags specified
	EDITSTYLEFLAG_DefaultOnly =				0x00000001,	//!< Only editing the "default" style
	EDITSTYLEFLAG_CAD =							0x00000002,	//!< Editing CAD styles
	EDITSTYLEFLAG_SingleType =					0x00000004,	//!< Only allow editing of specified style type
	EDITSTYLEFLAG_NoRenameStyles =			0x00000010,	//!< Don't allow styles to be renamed
	EDITSTYLEFLAG_NoNewDelStyles =			0x00000020,	//!< Don't allow styles to be created or deleted
	EDITSTYLEFLAG_NoElemHeight =				0x00000040,	//!< Don't include "Use element height" togglebutton
	EDITSTYLEFLAG_NoTypeButtons =				0x00000080,	//!< Don't include buttons for switching style type
	EDITSTYLEFLAG_NoScaleToLayout =			0x00000100,	//!< Don't allow scaling to "layout"
	EDITSTYLEFLAG_NoScaleOptions =			0x00000200,	//!< Don't give user options for scale-to
	EDITSTYLEFLAG_NoSample =					0x00000400,	//!< No not show "sample"
	EDITSTYLEFLAG_AllowScaleToNone =			0x00000800,	//!< Allow scaling to "none"
	EDITSTYLEFLAG_SymbolForceSquare =		0x00010000,	//!< Force symbol "width" to match "height"
	EDITSTYLEFLAG_SymbolNoNone =				0x00020000,	//!< Don't allow "no symbol" option
	EDITSTYLEFLAG_TextAllowHTML =				0x00100000,	//!< Allow HTML text formatting options
	EDITSTYLEFLAG_TextAllowSpecialFormat =	0x00200000,	//!< Allow "Special Format" options for text styles
	EDITSTYLEFLAG_TextAllowVertAlignment =	0x00400000,	//!< Add option for text vertical alignment
	EDITSTYLEFLAG_TextForceWordWrap =		0x00800000,	//!< Force word wrap
	EDITSTYLEFLAG_TextNoBlockWidth =			0x01000000,	//!< Don't allow setting block width
	EDITSTYLEFLAG_TextNoWordWrap =			0x02000000,	//!< Disallow setting word wrap
	};
#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(EDITSTYLEFLAGS)
#endif


//! User Interface form for editing style definition.
class FORM_EDIT_STYLE : public MGUI::FORM {
	public:

		//! Constructor.
		FORM_EDIT_STYLE (
			);

		//! Destructor.
		virtual ~FORM_EDIT_STYLE (
			);

		//! Commit changes made by user to file.
		ERRVALUE CommitChanges (
			);

		//! Create the form.
		ERRVALUE Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,
			RVC::STYLETYPE StyleType,
			EDITSTYLEFLAGS FormStyle = EDITSTYLEFLAG_None
			);

		//! Get active style number.
		//! @return Active style number, -1 for "default" style, < -1 if error.
		INT32 GetActiveStyleNum (
			) const { return (GetActiveStyleNum(m_StyleType)); }

		//! Get active style number for specified type.
		//! @return Active style number, -1 for "default" style, < -1 if error.
		INT32 GetActiveStyleNum (
			RVC::STYLETYPE StyleType
			) const;

		//! Get active style type.
		RVC::STYLETYPE GetActiveStyleType (
			) const { return (m_StyleType); }

		//! Get current scale.
		double GetCurrentScale (
			);

		//! Get default POINTSTYLE.
		void GetDefaultStyle (
			POINTSTYLE& PointStyle
			) const;

		//! Get default LINESTYLE.
		void GetDefaultStyle (
			LINESTYLE& LineStyle
			) const;

		//! Get default POLYSTYLE.
		void GetDefaultStyle (
			POLYSTYLE& PolyStyle
			) const;

		//! Get default TEXTSTYLE.
		void GetDefaultStyle (
			TEXTSTYLE& TextStyle
			) const;

		//! Get default MULTISTYLE.
		void GetDefaultStyle (
			MULTISTYLE& MultiStyle
			) const;

		//! Get current style object item.
		void GetStyleObjItem (
			RVC::OBJITEM& ObjItem
			) const;

		virtual bool IsCreated () const;

		//! Determine if RVC style object has been modified by user edits.
		bool IsObjectModified (
			) { CommitChanges(); return (m_ObjectModified); }

		//! Reset 'modified' status.
		void ResetModified (
			) { m_ObjectModified = false; }

		//! Set active style number for specified style type.
		//! This does not change the style type being "edited", use SetStyleType() to do that.
		ERRVALUE SetActiveStyleNum (
			RVC::STYLETYPE StyleType,
			INT32 StyleNum
			);

		//! Set current 'map scale' for style samples and 'Current Scale' button.
		void SetCurrentMapScale (
			double CurMapScale
			) { m_CurMapScale = CurMapScale; }

		//! Set default LINE style.
		void SetDefaultStyle (
			const LINESTYLE& LineStyle
			);

		//! Set default POINT style.
		void SetDefaultStyle (
			const POINTSTYLE& PointStyle
			);

		//! Set default POLYGON style.
		void SetDefaultStyle (
			const POLYSTYLE& PolygonStyle
			);

		//! Set default TEXT style.
		void SetDefaultStyle (
			const TEXTSTYLE& TextStyle
			);

		//! Set default MULTISTYLE style.
		void SetDefaultStyle (
			const MULTISTYLE& MultiStyle
			);

		//! Set delegate for obtaining current map scale.
		void SetDelegateGetCurrentScale (
			DELEGATE_DOUBLE_NOPARMS delegate
			) { m_DelegateGetCurrentScale = delegate; }
/*
		//! Set delegate for obtaining style description.
		void SetDelegateGetStyleDescription (
			FastDelegate<void(RVC::STYLETYPE,INT32,MISTRING&)> delegate
			) { m_DelegateGetStyleDescription = delegate; }
*/
		//! Set delegate to call when style edited.
		void SetDelegateNotifyModified (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateNotifyModified = delegate; }

		virtual bool IsEnabled () const;
		virtual void SetEnabled (bool enabled = true);

		//! Set hardcopy pixel spacing in "pixels per inch".
		//! This value is used when user chooses units to view style elements in.
		void SetHardcopyDPI (
			double HardcopyDPI
			) { m_HardcopyDPI = HardcopyDPI; }

		//! Set style object to edit or select styles from.
		ERRVALUE SetStyleObject (
			const RVC::OBJITEM& ObjItem,
			RVC::OBJECT::OPENMODE OpenMode = RVC::OBJECT::OPENMODE_Read
			);

		//! Set type of style being edited.
		void SetStyleType (
			RVC::STYLETYPE StyleType
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		class FORM_STYLES;
		friend class FORM_STYLES;
		class FORM_TYPE;
		friend class FORM_TYPE;
		class FORM_POINT;
		friend class FORM_POINT;
		class FORM_LINE;
		friend class FORM_LINE;
		class FORM_POLYGON;
		friend class FORM_POLYGON;
		class FORM_TEXT;
		friend class FORM_TEXT;

		//! Self-initializing members.
		MGUI::LAYOUT_PANE m_FormPane;
		MGUI::FORM_RADIOBUTTONS m_StyleTypeRB;
		MGUI::CTRL_EDIT_STRING m_SampleStrES;
		MGUI::IMAGELIST m_IconImageList;
		DELEGATE_DOUBLE_NOPARMS m_DelegateGetCurrentScale;
		DELEGATE_VOID_NOPARMS m_DelegateNotifyModified;

		//! Explicitly initialized.
		EDITSTYLEFLAGS m_FormStyleFlags;
		MDLGPARENT m_dlgparent;
		bool m_ObjectModified;
		UINT32 m_NoDrawSample;
		double m_HardcopyDPI;
		double m_CurMapScale;
		RVC::STYLE *m_pStyleObj;
		RVC::STYLETYPE m_StyleType;
		FORM_STYLES *m_pFormStyles;
		FORM_POINT *m_pFormPoint;
		FORM_LINE *m_pFormLine;
		FORM_POLYGON *m_pFormPolygon;
		FORM_TEXT *m_pFormText;
		FORM_TYPE *m_pFormType[RVC::STYLETYPE_NumTypes];
		int m_IconIdxCheckBoxEmpty;
		int m_IconIdxCheckBoxBlack;
		int m_IconIdxCheckBoxGray;

		ERRVALUE ChangeStyleObject ();
		bool DeletePattern (RVC::PATTERNTYPE PatternType, INT32 PatternNum);
		void EnableControls ();
		MDLGPARENT GetDlgParent () const { return (m_dlgparent); }
		double GetSizeUnitScale (int units) const;
		bool HasDftStyle () const { return (HasDftStyle(m_StyleType)); }
		bool HasDftStyle (RVC::STYLETYPE StyleType) const;
		bool HasStyle (RVC::STYLETYPE StyleType) const;
		bool InsertPatterns (RVC::PATTERNTYPE PatternType, INT32 PatternNum, const COLOR& varcolor);
		void InvalidateSample ();
		bool IsObjectWritable () const;
		ERRVALUE LoadStyle ();
		void NotifyModified ();
		void OnEditSampleStr ();
		void OnRadioStyleType ();
		void ResetStyleNums ();
		void RedrawStylesList ();
		INT32 SaveNewStyle (const RVC::OBJECTNAME& NewStyleName);
		void SetObjectModified (
			bool ObjectModified = true
			) { m_ObjectModified = ObjectModified; }

		// MGUI::FORM overrides.
		virtual void SetVisible (bool visible = true);

		FORM_EDIT_STYLE (const FORM_EDIT_STYLE&);
		FORM_EDIT_STYLE& operator= (const FORM_EDIT_STYLE&);

		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//!\addtogroup DialogFunctions 
//!@{


//! Dialog allowing selection of single icon from style object.
//! Same as DlgGetPattern except that it assumes Bitmap patterns, and uses
//!	icons.ref instead of stdstyle.rvc for the "standard" styles.
//! @return Pattern index or error < 0
int DlgGetIcon (
	MDLGPARENT dlgparent,					//!< Parent for dialog
	const MISTRING& PromptStr,				//!< Prompt to show at top of dialog
	RVC::OBJITEM& StyleObjItem,			//!< Style object item the icon was selected from
	const COLOR& SampleColor				//!< "Variable" color for sample displays
	);

//! Dialog allowing selection of single pattern from style object.
//! @return Pattern index or error < 0
int DlgGetPattern (
	MDLGPARENT dlgparent,					//!< Parent for dialog
	const MISTRING& PromptStr,				//!< Prompt to show at top of dialog
	RVC::PATTERNTYPE PatternType,		//!< Type of pattern
	const RVC::OBJITEM& StyleObjItem,	//!< Style object item to select pattern from
	const COLOR& SampleColor				//!< "Variable" color for sample displays
	);


//! Dialog allowing selection of patterns from style object.
ERRVALUE DlgGetPatterns (
	MDLGPARENT dlgparent,					//!< Parent for dialog
	const MISTRING& PromptStr,				//!< Prompt to show at top of dialog
	RVC::PATTERNTYPE PatternType,		//!< Type of pattern
	RVC::OBJITEM& StyleObjItem,			//!< Style object item passed/returned
	SIMPLE_ARRAY<INT32>& PatternNums,	//!< Patterns selected
	const COLOR& SampleColor				//!< "Variable" color for sample displays
	);


//! Dialog to edit styles in style object.
ERRVALUE DlgStyleEdit (
	MDLGPARENT dlgparent,					//!< Parent for dialog
	const RVC::OBJITEM& ObjItemStyle,	//!< Style object item
	double CurMapScale,						//!< Map scale to set when 'Current' button pressed
	EDITSTYLEFLAGS flags,					//!< Flags
	RVC::STYLETYPE StyleType = STYLETYPE_NumTypes	//!< Type of style to edit, TYPE_NumTypes for all
	);


//! Dialog to edit point style(s) including 'default'.
ERRVALUE DlgStyleEdit (
	MDLGPARENT dlgparent,					//!< Parent for dialog
	const RVC::OBJITEM& ObjItemStyle,	//!< Style object item
	POINTSTYLE& DftStyle,					//!< 'Default' style passed/returned
	double CurMapScale,						//!< Map scale to set when 'Current' button pressed
	EDITSTYLEFLAGS flags						//!< Flags
	);


//! Dialog to edit line style(s) including 'default'.
ERRVALUE DlgStyleEdit (
	MDLGPARENT dlgparent,					//!< Parent for dialog
	const RVC::OBJITEM& ObjItemStyle,	//!< Style object item
	LINESTYLE& DftStyle,						//!< 'Default' style passed/returned
	double CurMapScale,						//!< Map scale to set when 'Current' button pressed
	EDITSTYLEFLAGS flags						//!< Flags
	);


//! Dialog to edit polygon style(s) including 'default'.
ERRVALUE DlgStyleEdit (
	MDLGPARENT dlgparent,					//!< Parent for dialog
	const RVC::OBJITEM& ObjItemStyle,	//!< Style object item
	POLYSTYLE& DftStyle,						//!< 'Default' style passed/returned
	double CurMapScale,						//!< Map scale to set when 'Current' button pressed
	EDITSTYLEFLAGS flags						//!< Flags
	);


//! Dialog to edit text style(s) including 'default'.
ERRVALUE DlgStyleEdit (
	MDLGPARENT dlgparent,					//!< Parent for dialog
	const RVC::OBJITEM& ObjItemStyle,	//!< Style object item
	TEXTSTYLE& DftStyle,						//!< 'Default' style passed/returned
	double CurMapScale,						//!< Map scale to set when 'Current' button pressed
	EDITSTYLEFLAGS flags						//!< Flags
	);


//! Dialog to edit text style(s) including 'default'.
ERRVALUE DlgStyleEdit (
	MDLGPARENT dlgparent,					//!< Parent for dialog
	const RVC::OBJITEM& ObjItemStyle,	//!< Style object item
	MULTISTYLE& DftStyle,					//!< 'Default' style passed/returned
	double CurMapScale,						//!< Map scale to set when 'Current' button pressed
	EDITSTYLEFLAGS flags,						//!< Flags
	RVC::STYLETYPE StyleType = STYLETYPE_NumTypes	//!< Type of style to edit, TYPE_NumTypes for all
	);


//! Dialog allowing "pattern" in style object to be edited.
ERRVALUE DlgStyleEditPattern (
	MDLGPARENT dlgparent,					//!< Parent for dialog
	const RVC::OBJITEM& StyleObjItem,	//!< Style object item
	RVC::PATTERNTYPE PatternType,			//!< Type of pattern
	INT32 PatternNum,							//!< Pattern number to edit, -1 for "new" pattern
	const COLOR& varcolor					//!< "Variable" color for sample displays
	);


//! Dialog allowing "pattern set" in style object to be edited.
ERRVALUE DlgStyleEditPatternSet (
	MDLGPARENT dlgparent,					//!< Parent for dialog
	const RVC::OBJITEM& StyleObjItem,	//!< Style object item
	RVC::PATTERNTYPE PatternType,			//!< Type of pattern
	const COLOR& varcolor,					//!< "Variable" color for sample displays
	INT32 PatternNum = -1					//!< Default pattern number
	);


//! Dialog for editing 'style assignment' table for point style.
//! @return TRUE if table modified, FALSE if not, < 0 if error.
int DlgTableEditStyleByAttribute (
	MDLGPARENT dlgparent,					//!< Parent for dialog
	const RVC::OBJITEM& dbobjitem,		//!< Database object item reference
	ELEMTYPE elemtype,						//!< Element type
	RVC::OBJECTNAME& tablename,			//!< Style assignment table name passed/returned
	POINTSTYLE& DftStyle,					//!< 'Default' style passed/returned
	double CurMapScale,						//!< Map scale to set when 'Current' button pressed
	const OBJITEM *pDftStyleObjItem = 0	//!< Default style object if any
	);


//! Dialog for editing 'style assignment' table for line style.
//! @return TRUE if table modified, FALSE if not, < 0 if error.
int DlgTableEditStyleByAttribute (
	MDLGPARENT dlgparent,					//!< Parent for dialog
	const RVC::OBJITEM& dbobjitem,		//!< Database object item reference
	ELEMTYPE elemtype,						//!< Element type
	RVC::OBJECTNAME& tablename,			//!< Style assignment table name passed/returned
	LINESTYLE& DftStyle,						//!< 'Default' style passed/returned
	double CurMapScale,						//!< Map scale to set when 'Current' button pressed
	const OBJITEM *pDftStyleObjItem = 0	//!< Default style object if any
	);


//! Dialog for editing 'style assignment' table for polygon style.
//! @return TRUE if table modified, FALSE if not, < 0 if error.
int DlgTableEditStyleByAttribute (
	MDLGPARENT dlgparent,					//!< Parent for dialog
	const RVC::OBJITEM& dbobjitem,		//!< Database object item reference
	ELEMTYPE elemtype,						//!< Element type
	RVC::OBJECTNAME& tablename,			//!< Style assignment table name passed/returned
	POLYSTYLE& DftStyle,						//!< 'Default' style passed/returned
	double CurMapScale,						//!< Map scale to set when 'Current' button pressed
	const OBJITEM *pDftStyleObjItem = 0	//!< Default style object if any
	);


//! Dialog for editing 'style assignment' table for MULTISTYLE.
//! @return TRUE if table modified, FALSE if not, < 0 if error.
int DlgTableEditStyleByAttribute (
	MDLGPARENT dlgparent,					//!< Parent for dialog
	const RVC::OBJITEM& dbobjitem,		//!< Database object item reference
	RVC::OBJECTNAME& tablename,			//!< Style assignment table name passed/returned
	MULTISTYLE& DftStyle,					//!< 'Default' style passed/returned
	double CurMapScale,						//!< Map scale to set when 'Current' button pressed
	const OBJITEM *pDftStyleObjItem = 0,//!< Default style object if any
	RVC::STYLETYPE DftStyleType = RVC::STYLETYPE_NumTypes	//!< Default style type
	);


//! Dialog for editing 'style assignment' table for style by theme.
//! @return TRUE if table modified, FALSE if not, < 0 if error.
int DlgTableEditStyleByTheme (
	MDLGPARENT dlgparent,					//!< Parent for dialog
	const RVC::OBJITEM& dbobjitem,		//!< Database object item reference
	ELEMTYPE elemtype,						//!< Element type
	RVC::OBJECTNAME& tablename				//!< Style assignment table name passed/returned
	);

//!@}

}	// End namespace UI
}	// End namespace RVC

#endif	// INC_RVC_UISTYLE_H
