/******************************************************************************
 *
 * \file sml/tinobject.h
 * \brief SML TINOBJECT
 *
 * \if NODOC
 * $Log: tinobject.h_v $
 * Revision 1.2  2007/05/08 15:46:31  dwilliss
 * SML namespace
 *
 * Revision 1.1  2007/05/04 21:07:20  dwilliss
 * Initial revision
 *
 * \endif
*/
/*****************************************************************************/

#ifndef INC_SML_TINOBJECT_H
#define INC_SML_TINOBJECT_H

#ifndef INC_SML_OBJECT_H
#include <sml/object.h>
#endif

#ifdef RVCSYSDLL
	#define RVCSYSLIBEXPORT MI_DLLEXPORT
	#define RVCSYSLIBCLASSEXPORT MI_DLLCLASSEXPORT
#else
	#define RVCSYSLIBEXPORT MI_DLLIMPORT
	#define RVCSYSLIBCLASSEXPORT MI_DLLCLASSIMPORT
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
struct SMLRASTBUF;
namespace RVC {
	class TIN;
	class TINTOOLKIT;
	class TINHANDLE;
	}
#endif

namespace SML {

class RVCSYSLIBCLASSEXPORT TINOBJECT : public OBJECT {
	public:

		//! Get a pointer to the RVC::TIN interface class.
		//! Do NOT free this pointer.  It will be destroyed when the SML::OBJECT is
		//! destroyed.  The interface returned is Attached to the object but may
		//! or may not be open.
		//! If the SML::OBJECT does not refer to any existing real object, this
		//! method will return NULL.
		RVC::TIN* GetInterface (
			) const;

		RVC::TINTOOLKIT* GetToolkitHandle (
			) const;			//!<  used for writing to TIN

		//! Set a pointer to the RVC::TIN interface class.
		//! After setting the interface, the SML::OBJECT "owns" it and will 
		//! delete it in its own destructor.  Do not under any circumstances
		//! pass a pointer to an local variable that will go out of scope.
		//! Since SML::OBJECT is implemented in rvcsys.dll it does not have
		//! have access to the copy constructor.
		void SetInterface (
			RVC::TIN* obj
			);

		void SetToolkitHandle (
			RVC::TINTOOLKIT* ttk
			);

	private:
#ifndef GENERATING_DOXYGEN_OUTPUT
		// Constructor
		TINOBJECT (
			CONTEXT* context
			);

		// Destructor
		virtual ~TINOBJECT (
			);

		virtual int v_OpenLow (
			int mode
			);

		virtual void v_CloseLow (
			);

		virtual RVC::OBJTYPE v_GetObjType (
			) const;

		RVC::TINHANDLE* GetHandle (
			) const { return m_handle; }

		RVC::TINTOOLKIT* m_TINToolkitHandle;	//!<  Used for writing to TIN
		RVC::TINHANDLE* m_handle;

		friend class CONTEXT;		//!< Only SML::CONTEXT can call our ctor/dtor
		friend class OBJECT;			//!< All our virtual methods are private and called through SML::OBJECT
		friend class VM;			//!< Virtual Machine needs access to otherwise private methods
#endif // GENERATING_DOXYGEN_OUTPUT
	};

} // end of SML namespace

typedef SML::TINOBJECT SMLTINOBJECT;

#undef RVCSYSLIBEXPORT
#undef RVCSYSLIBCLASSEXPORT

#endif		//  INC_SML_TINOBJECT_H


