/**
 * \file imagefilterreduce2.h <spatmod/imagefilterreduce2.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_REDUCE2 definition.
 *
 * \if NODOC
 * $Id: imagefilterreduce2.h_v 1.1 2010/12/23 18:31:35 mju Exp $
 *
 * $Log: imagefilterreduce2.h_v $
 * Revision 1.1  2010/12/23 18:31:35  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERREDUCE2_H
#define	INC_SPATMOD_IMAGEFILTERREDUCE2_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

#ifndef	INC_SPATMOD_IMAGESECTION_H
#include <spatmod/imagesection.h>
#endif

#ifndef	INC_SPATMOD_IMAGERESAMPLEMETHOD_H
#include <spatmod/imageresamplemethod.h>
#endif

namespace SPATMOD {
namespace IMAGE {

class BUFFER;

//=====================================================================================================================

//! Filter perform 2X image reduction.
class FILTER_REDUCE2 : public FILTER {
	public:

		//! Constructor.
		FILTER_REDUCE2 (
			STAGE *pPreviousStage,					//!< Pointer to previous stage
			bool TakeOwnership,						//!< Filter takes ownership of previous stage and will delete when needed
			RESAMPLEMETHOD ResampleMethod			//!< Resampling method
			);

		// Destructor.
		virtual ~FILTER_REDUCE2 ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		RESAMPLEMETHOD m_ResampleMethod;
		INT32 m_Margin;
		INT32 m_MarginB;
		INT32 m_MarginR;
		SECTION m_PrevSection;
		INT16 m_Kernel4[4][4];
		INT32 m_MinCount;

		ERRVALUE BeginIteratorPrevStage (
			const SECTION_ITERATOR& Iterator,
			SECTION_ITERATOR& IteratorPrev
			) const;

		void CalcKernelBicubic (INT16 v1, INT16 v2);
		ERRVALUE DoAverage (BUFFER& TgtBuffer, BUFFER& SrcBuffer) const;
		ERRVALUE DoAverageUINT8 (BUFFER& TgtBuffer, const BUFFER& SrcBuffer) const;
		ERRVALUE DoAverageSINT8 (BUFFER& TgtBuffer, const BUFFER& SrcBuffer) const;
		ERRVALUE DoAverageUINT16 (BUFFER& TgtBuffer, const BUFFER& SrcBuffer) const;
		ERRVALUE DoAverageSINT16 (BUFFER& TgtBuffer, const BUFFER& SrcBuffer) const;
		ERRVALUE DoAverageUINT32 (BUFFER& TgtBuffer, const BUFFER& SrcBuffer) const;
		ERRVALUE DoAverageSINT32 (BUFFER& TgtBuffer, const BUFFER& SrcBuffer) const;
		ERRVALUE DoAverageFLOAT32 (BUFFER& TgtBuffer, const BUFFER& SrcBuffer) const;
		ERRVALUE DoAverageFLOAT64 (BUFFER& TgtBuffer, const BUFFER& SrcBuffer) const;

		ERRVALUE DoBicubic (BUFFER& TgtBuffer, BUFFER& SrcBuffer) const;
		ERRVALUE DoBicubicUINT8 (BUFFER& TgtBuffer, const BUFFER& SrcBuffer) const;
		ERRVALUE DoBicubicSINT8 (BUFFER& TgtBuffer, const BUFFER& SrcBuffer) const;
		ERRVALUE DoBicubicUINT16 (BUFFER& TgtBuffer, const BUFFER& SrcBuffer) const;
		ERRVALUE DoBicubicSINT16 (BUFFER& TgtBuffer, const BUFFER& SrcBuffer) const;
		ERRVALUE DoBicubicUINT32 (BUFFER& TgtBuffer, const BUFFER& SrcBuffer) const;
		ERRVALUE DoBicubicSINT32 (BUFFER& TgtBuffer, const BUFFER& SrcBuffer) const;
		ERRVALUE DoBicubicFLOAT32 (BUFFER& TgtBuffer, const BUFFER& SrcBuffer) const;
		ERRVALUE DoBicubicFLOAT64 (BUFFER& TgtBuffer, const BUFFER& SrcBuffer) const;

		ERRVALUE DoMaximum (BUFFER& TgtBuffer, BUFFER& SrcBuffer) const;
		ERRVALUE DoMaximumUINT8 (BUFFER& TgtBuffer, const BUFFER& SrcBuffer) const;
		ERRVALUE DoMaximumSINT8 (BUFFER& TgtBuffer, const BUFFER& SrcBuffer) const;
		ERRVALUE DoMaximumUINT16 (BUFFER& TgtBuffer, const BUFFER& SrcBuffer) const;
		ERRVALUE DoMaximumSINT16 (BUFFER& TgtBuffer, const BUFFER& SrcBuffer) const;
		ERRVALUE DoMaximumUINT32 (BUFFER& TgtBuffer, const BUFFER& SrcBuffer) const;
		ERRVALUE DoMaximumSINT32 (BUFFER& TgtBuffer, const BUFFER& SrcBuffer) const;
		ERRVALUE DoMaximumFLOAT32 (BUFFER& TgtBuffer, const BUFFER& SrcBuffer) const;
		ERRVALUE DoMaximumFLOAT64 (BUFFER& TgtBuffer, const BUFFER& SrcBuffer) const;

		ERRVALUE DoMinimum (BUFFER& TgtBuffer, BUFFER& SrcBuffer) const;
		ERRVALUE DoMinimumUINT8 (BUFFER& TgtBuffer, const BUFFER& SrcBuffer) const;
		ERRVALUE DoMinimumSINT8 (BUFFER& TgtBuffer, const BUFFER& SrcBuffer) const;
		ERRVALUE DoMinimumUINT16 (BUFFER& TgtBuffer, const BUFFER& SrcBuffer) const;
		ERRVALUE DoMinimumSINT16 (BUFFER& TgtBuffer, const BUFFER& SrcBuffer) const;
		ERRVALUE DoMinimumUINT32 (BUFFER& TgtBuffer, const BUFFER& SrcBuffer) const;
		ERRVALUE DoMinimumSINT32 (BUFFER& TgtBuffer, const BUFFER& SrcBuffer) const;
		ERRVALUE DoMinimumFLOAT32 (BUFFER& TgtBuffer, const BUFFER& SrcBuffer) const;
		ERRVALUE DoMinimumFLOAT64 (BUFFER& TgtBuffer, const BUFFER& SrcBuffer) const;

		// BASE/STAGE/FILTER overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullBegin (const SECTION_ITERATOR& iterator);
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);
		virtual float v_QueryIterator (const SECTION_ITERATOR& iterator) const;

		// Unimplemented
		FILTER_REDUCE2 (const FILTER_REDUCE2&);
		FILTER_REDUCE2& operator= (const FILTER_REDUCE2&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERREDUCE2_H

