/**
 * \file spatmod/imageresamplemethod.h
 * \brief SPATMOD::IMAGE::RESAMPLEMETHOD enumeration definitions.
 *
 * \if NODOC
 * $Id: imageresamplemethod.h_v 1.4 2010/12/01 14:19:20 mju Exp $
 *
 * $Log: imageresamplemethod.h_v $
 * Revision 1.4  2010/12/01 14:19:20  mju
 * Add Lanczos.
 *
 * Revision 1.3  2010/11/30 16:00:07  mju
 * Add bicubicSmoother/Sharper.
 *
 * Revision 1.2  2010/11/29 18:35:37  mju
 * Add bicubic, make cubicconvolution alias to it.
 *
 * Revision 1.1  2006/10/16 17:49:26  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGERESAMPLEMETHOD_H
#define	INC_SPATMOD_IMAGERESAMPLEMETHOD_H

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Resampling method enumeration used in some filters.
enum RESAMPLEMETHOD {
	RESAMPLEMETHOD_Nearest = 0,					//!< Nearest neighbor
	RESAMPLEMETHOD_Bilinear = 1,					//!< Bilinear interpolation
	RESAMPLEMETHOD_Bicubic = 2,					//!< Bicubic interpolation
	RESAMPLEMETHOD_Average = 3,					//!< Average (for res-down only)
	RESAMPLEMETHOD_Minimum = 4,					//!< Minimum sample values (for res-down only)
	RESAMPLEMETHOD_Maximum = 5,					//!< Maximum sample values (for res-down only)
	RESAMPLEMETHOD_BicubicSharper = 6,
	RESAMPLEMETHOD_BicubicSmoother = 7,
	RESAMPLEMETHOD_Lanczos2 = 8,					//!< Lanczos with a=2 (4x4 grid of points)
//	RESAMPLEMETHOD_Lanczos2 = 9,					//!< Lanczos with a=3 (6x6 grid of points)
	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGERESAMPLEMETHOD_H
