/**
 * toolskel - Example implementation of a GRE_TOOL
**/

#if defined(X_NATIVE)		// X only

#include <gre/viewlegacy.h>
#include <mi32/xdefns.h>
#include "toolskel.h"

// Class definition.
class GRE_TOOL_EXAMPLE : public GRE::TOOL {
	public:

		GRE_TOOL_EXAMPLE (
			GRE::VIEW_LEGACY *view
			);

		virtual ~GRE_TOOL_EXAMPLE ();

	private:

		static void EH_DrawingArea (
			Widget,
			GRE_TOOL_EXAMPLE *tool,
			XEvent *event,
			Boolean *
			);


		void OnMouseDown (
			int button,			// Button1 == left, Button2 == middle, Button3 == right
			UINT32 state,
			int x,
			int y
			);

		void OnMouseUp (
			int button,			// Button1 == left, Button2 == middle, Button3 == right
			UINT32 state,
			int x,
			int y
			);

		void OnMouseMove (
			int button,			// Button1 == left, Button2 == middle, Button3 == right
			UINT32 state,
			int x,
			int y
			);

		void OnKeyPress (
			const char* key,
			UINT32 state
			);


		virtual bool DoActivate ();
		virtual void DoDeactivate ();
		virtual bool DoInitialize ();
		virtual void DoSuspend ();
		virtual void DoResume ();

		MXXTEH_ADD(GRE_TOOL_EXAMPLE)
		MXXTEH_REMOVE(GRE_TOOL_EXAMPLE)

		GRE_TOOL_EXAMPLE (const GRE_TOOL_EXAMPLE&);
		GRE_TOOL_EXAMPLE& operator= (const GRE_TOOL_EXAMPLE&);
	};	//:> End of GRE_TOOL_EXAMPLE


// Constructor.
GRE_TOOL_EXAMPLE::GRE_TOOL_EXAMPLE (
	GRE::VIEW_LEGACY *view
	) :
	// Base		  tool-tip       icon name    icon group
	GRE::TOOL(view,"ExampleTool","edit_block","standard")
	{
	}


// Destructor
GRE_TOOL_EXAMPLE::~GRE_TOOL_EXAMPLE (
	) {
	// Remove our event handler or we'll crash
	MxXtEventHandlerRemove(m_view->GetDrawingArea(),XtAllEvents,EH_DrawingArea);
	// Clean up anything else here
	}


// Event handler for DrawingArea
void GRE_TOOL_EXAMPLE::EH_DrawingArea (
	Widget,
	GRE_TOOL_EXAMPLE *tool,
	XEvent *event,
	Boolean * ContinueToProcess
	) {
	switch (event->type) {
		case ButtonPress: 
			tool->OnMouseDown(event->xbutton.button, event->xbutton.state, event->xbutton.x, event->xbutton.y);
			break;

		case ButtonRelease:
			tool->OnMouseUp(event->xbutton.button, event->xbutton.state, event->xbutton.x, event->xbutton.y);

		case EnterNotify:
			// Mouse has entered the drawing area
			break;

		case LeaveNotify:
			// Mouse has left the drawing area
			break;

		case MotionNotify: {
			int button = 0;
			if (event->xmotion.state & Button1Mask) button = 1;
			else if (event->xmotion.state & Button2Mask) button = 2;
			else if (event->xmotion.state & Button3Mask) button = 3;
			tool->OnMouseMove(button, event->xmotion.state, event->xmotion.x, event->xmotion.y);
			}
			break;

		case KeyPress: {
			char keystring[4];
			XLookupString(&event->xkey,keystring,1,NULL,NULL);
			tool->OnKeyPress(keystring, event->xkey.state);
			break;
			}
		}
	return;
	}


// Called once when the tool is first created but before it's active
bool GRE_TOOL_EXAMPLE::DoInitialize (
	) {
	return (true);
	}


// Called when the user clicks the button to activate your tool
bool GRE_TOOL_EXAMPLE::DoActivate (
	) {
	// Setup our event handler
	MxXtEventHandlerAdd(m_view->GetDrawingArea(),ButtonPressMask|ButtonReleaseMask|ButtonMotionMask|EnterWindowMask|LeaveWindowMask|PointerMotionMask|KeyPressMask,EH_DrawingArea);
	return (true);
	}


// Called when another tools is activated and yours is deactivated
void GRE_TOOL_EXAMPLE::DoDeactivate (
	) {
	// Remove our event handler (very important.  Will crash otherwise)
	MxXtEventHandlerRemove(m_view->GetDrawingArea(),XtAllEvents,EH_DrawingArea);
	return;
	}


// Called just after a redraw
void GRE_TOOL_EXAMPLE::DoResume (
	) {
	return;
	}


// Called just before a redraw
void GRE_TOOL_EXAMPLE::DoSuspend (
	) {
	return;
	}


// Called when the mouse button is pressed
void GRE_TOOL_EXAMPLE::OnMouseDown (
	int button,			// Button1 == left, Button2 == middle, Button3 == right
	UINT32 state,
	int x,				// Mouse location in "screen" coordinates relative to
	int y					//	  the upper left of the drawing area
	) {
	return;
	}


// Called when the mouse button is released
void GRE_TOOL_EXAMPLE::OnMouseUp (
	int button,			// Button1 == left, Button2 == middle, Button3 == right
	UINT32 state,
	int x,				// Mouse location in "screen" coordinates relative to
	int y					//	  the upper left of the drawing area
	) {
	return;
	}


// Called when the mouse moves (button up or down)
void GRE_TOOL_EXAMPLE::OnMouseMove (
	int button,			// 0==none, Button1==left, Button2==middle, Button3==right
	UINT32 state,
	int x,				// Mouse location in "screen" coordinates relative to
	int y					//	  the upper left of the drawing area
	) {
	return;
	}


// Called when key is pressed.
void GRE_TOOL_EXAMPLE::OnKeyPress (
	const char* key,
	UINT32 state
	) {
	return;
	}

/**
*	GRE_ToolAddExample - Add the example tool.  Rename as desired
*
*		You should call this after calling MdispViewCreate and before
*		you call view->AddToolIcons()
**/

void GLOBALFUNC GRE_ToolAddExample (
	GRE::VIEW_LEGACY *view
	) {
	new GRE_TOOL_EXAMPLE(view);
	// Just constructing the GRE_TOOL causes it to register itself so that 
	// view->AddToolIcons() will know about it.  The tool will be deleted
	// by the view's destructor, so you should never "delete" it yourself.
	}

#endif	// X only
